# -*- coding: utf-8 -*-
"""
Created on Wed Nov  3 12:38:18 2021

@author: ScottStation
"""

import time
from .qecontext import qeContextBase
from .qetype import qetype
from datetime import datetime,timedelta
from .qeglobal import  is_trade_day
from .qectpmarket_wrap import changeCtpInstIDs
#from .qesoptmarket import changeSoptInstIDs
#from .qesoptmarket import changeSoptInstIDs
from .qeredisdb import saveHedgeMarketToDB
#from .qeredisdb import saveStrategyContextToDB
#from .qeredisdb import saveOrderDatarealToDB,saveOrderDataToDB,saveTradeDatarealToDB,savePositionDatarealToDB
from .qeredisdb import saveTradingDay
from .qeredisdb import loadSettingData, loadSettingDatareal, saveSettingDataToDB, saveSettingDatarealToDB
#from .qeredisdb import saveStrategyfreqToDB,saveStrategyfreqrealToDB
from .qelogger import logger
from threading import Timer
#from .qectpmarket import checkMarketTime
#from .qecontext import transInstID2Real,transInstID2Context
#from .qeaccount import realaccount
from .qesimtrader import simuaccount
#from .qeriskctl import soptriskctl
from .qeinterface import cancel_order, make_order
from .qeglobal import g_userinfo, qeInstSett, instSetts, setTradingDaySaved, getTradingDaySaved,getPositionLoaded
import random
from .qestratmarket_wrap import readStratPosition_wrap, writeStratPosition_wrap,writeStratStat_wrap,writeStratTrade_wrap
from .qestratmarket_wrap import readStratStat_wrap,writeContract_messages_wrap,writeContractTable_wrap
try:
    from .qestockmarket import changeStockInstIDs
except:
    changeStockInstIDs = lambda x,y:None

def formhedgeInstid(instid):
    for i in range(len(instid)):
        if i == 0:
            instid_hedge = str(instid[i])+'_'
        else:
            instid_hedge += str(instid[i])+'_'
    instid_hedge = instid_hedge[:-1]
    return instid_hedge
        
        
def getBarTime(minu, freq):
    tday = datetime.today()
    
    try:
        curminu = int(minu) - 2400 if int(minu) >= 2400 else int(minu)
        timestr = tday.strftime('%Y-%m-%d') + ' '+"{:04d}".format(curminu)+'00'
        bartime = datetime.strptime(timestr, '%Y-%m-%d %H%M%S')
        if freq == 1:
            return bartime
        else:    
            startday = tday if int(minu) < 2400 else tday + timedelta(days=1)
            starttime = datetime.strptime(startday.strftime('%Y-%m-%d') + ' 210000','%Y-%m-%d %H%M%S')
            if int((bartime - starttime).seconds/60) % freq == 0:
                return bartime
            else:
                return None
    except:
        now = datetime.now()
        curminu = int(now.hour*100+now.minute)
        curminu = int(minu) - 2400 if int(minu) >= 2400 else int(minu)
        timestr = tday.strftime('%Y-%m-%d') + ' '+str(curminu)+'00'
        bartime = datetime.strptime(timestr, '%Y-%m-%d %H%M%S')
        if freq == 1:
            return bartime
        else:    
            startday = tday if int(minu) < 2400 else tday + timedelta(days=1)
            starttime = datetime.strptime(startday.strftime('%Y-%m-%d') + ' 210000','%Y-%m-%d %H%M%S')
            if int((bartime - starttime).seconds/60) % freq == 0:
                return bartime
            else:
                return None
            

def getDailyCloseTime(instid, leftminu):
    now = datetime.now()
    endtime = []
    for inst in instid:
        if inst[-3:] == 'SGE':
            endtime.append(1530)
        elif inst[0] == 'T':
            endtime.append(1515)
        
        else:
            endtime.append(1500)
    endt = min(endtime)
    endday=now.strftime("%Y-%m-%d ")
    return datetime.strptime(endday+str(endt)+'00', '%Y-%m-%d %H%M%S') - timedelta(minutes=leftminu)
            
class qeStratProcess:
    def __init__(self, name, Queue, realTrade=False):

        self.instid = []          # list of instrument
        #self.instid_ex = []
        #self.exID = []
        self.stratQueue = Queue
        #self.tqueue = traderQ
        self.name = name
        self.realTrade = realTrade
        self.referenceID = datetime.now().strftime('%H%M%S')+'0000'
        self.time_delay = 0   
        self.formula = None
        self.hedgemodel = False
        self.printlog = True
        self.hedgemodel_price = 0
        self.hedgemodel_time = 0
        self.user = "unknown"
        self.token = ''
        self.Timer_interval = 30
        self.work = True
        self.trigger = False
        self.current_timedigit = 0
        self.balance=0
        self.evalmode = False
        self.daily_called = False
        #bar
        self.wait_all=True #
        #self.freq=0 #0为tick
        #self.info_time_new=0
        #self.info_time_all=0
        self.info_time = 0
        self.posLoaded = False
        
        self.bDataRead = False
        self.bCross = False
        #self.saveTradingday = False
    def getLocalTradingDay(self):
        tday = datetime.now()
        if tday.hour > 18:
            days = 1 if tday.weekday() < 4 else 3
            tday += timedelta(days=days)
        elif tday.hour < 8 and tday.weekday()==5:
            tday += timedelta(days=2)
        return tday

    def checkCrossDay(self):
        if self.bDataRead:
            now = datetime.now()
            ##set crossDay True
            if not self.bCross and now.hour == 19:
                self.bCross = True
                return True
            ##restore crossDay to False
            elif self.bCross and now.hour > 19:
                self.bCross = False
                
        return False
    
    def stratProcess(self, strat):
        global info_time
        print(u"策略进程启动成功；"+str(strat.name))
        logger.info(u"策略进程启动成功；"+str(strat.name))
        logger.info("strat user "+str(self.user))
        context = qeContextBase(self.instid)
        #g_userinfo = globalInfo()  
        #print(g_userinfo.info_time,'#')
        #print(1)
        
        context.user = self.user
        context.token = self.token
        context.hedgemodel = self.hedgemodel
        context.printlog = self.printlog
        context.trader = self.trader
        context.runmode = 'simu' if self.trader =='simu' else 'real'
        if self.hedgemodel:
            context.formula = self.formula
            self.instid_hedge = formhedgeInstid(self.instid)

            
        context.init_orderid = int(str(self.ID)+ self.referenceID)
        context.orderid = context.init_orderid
        context.freq=strat.freq
        #context.instid = self.instid #self.instid_ex  
        context.stratName = self.name
        context.realTrade = self.realTrade
        context.feesMult = self.feesmult
        if self.realTrade:
            context.accounts = self.accounts
            context.account = context.accounts[0]
        else:
            context.account = simuaccount
            context.accounts = [simuaccount]
        if self.recmode:
            simuaccount.loadPosition = False
            context.recmode = True
            
        #print(3,g_userinfo.info_time_all)
        #print(2,self.freq)
        #print(1,self.wait_all)
        #strat.initStrat(context)
        #context.freq=self.freq
        #print('111',info_time)
        if self.evalmode:
            self.Timer_interval = 8
        self.callTimer()

        if self.stratQueue:
            while True:
                #print("strat:", self.name)
                while not self.stratQueue.empty():
                    #print(123)
                    try:
                        d = self.stratQueue.get(block = True, timeout = 1)
                        #print(d)
                        if d['type'] == qetype.KEY_MARKET_DATA or d['type'] == qetype.KEY_MARKET_MULTIDATA:         
#                             print('stratprocess market data')
                            savedInsts = strat.instid.copy();
                            if d['type'] == qetype.KEY_MARKET_DATA:
                                self.updateData(d,context)
                                self.updateTime(d, context)
                            elif d['type'] == qetype.KEY_MARKET_MULTIDATA:
                                for data in d['data']:
                                    self.updateData(data,context)
                                self.updateTime(d['data'][0], context)
                            #self.autoOrders(d, context)
                            #self.algoTrade(d,context)
                                #if context.runmode == 'simu':
                                #    context.account.noticeCrossDay(self.name)
                            #self.refresh(d,context) 
                            #if isinstance(context.trader, list)  and 'stockoption' in context.trader:
                            #    retryorders =  soptriskctl.getRetryOrders()
                            #    if retryorders:
                            #        for oid in retryorders:
                            #            cancel_order(context, oid)
                            if context.runmode=='real' and not getPositionLoaded():
                                continue
                            try:
                                    #print(1)
                                    #print(self.wait_all)
                                    

                                strat.handleData(context)
                                if strat.datamode=='minute':
                                    renew = False
                                    if self.wait_all and self.info_time != g_userinfo.info_time_all:
                                        self.info_time = g_userinfo.info_time_all
                                        renew = True
                                    elif not self.wait_all and  self.info_time != g_userinfo.info_time_new : 
                                        self.info_time = g_userinfo.info_time_new 
                                        renew = True
                                    if renew:
                                        bartime = getBarTime(self.info_time, strat.freq)
                                        if bartime:
                                            context.bartime = bartime
                                            strat.onBar(context)
                                    
                                elif strat.datamode == 'daily':
                                    if not  self.daily_called and context.curtime >= getDailyCloseTime(strat.instid, strat.dailyminutes):
                                        self.daily_called = True
                                        strat.onBar(context)
                                #strat.instid = getValidInstIDs(strat.instid)
                                if savedInsts !=  strat.instid:
                                    print("instid changed from ", savedInsts, 'to', strat.instid)
                                    self.changeInstIDs(context, savedInsts, strat.instid)
                                    context.instid = strat.instid
#                                         self.trigger = False
                            except Exception as e:
                                logger.error(f'Error on strat.handleData {e}',exc_info=True)
                        elif d['type'] == qetype.KEY_ON_ORDER:
                            #print('on_Order')
                            #print('3',d)
                            self.handleOrder(d,context)
                            try:
                                strat.handleOrder(d,context)
                            except Exception as e:
                                print(f'Error on strat.handleOrder: {e}')
                        elif d['type'] == qetype.KEY_ON_ORDER_ERROR:
                            #print('on_Order')
                            #print('3',d)
                            self.handleOrder(d,context)
                            try:
                                strat.handleOrderError(d,context)
                            except Exception as e:
                                print(f'Error on strat.handleOrderError: {e}')

                        elif d['type'] == qetype.KEY_ON_TRADE:
                            try:
                                strat.handleTrade(d,context)
                            except Exception as e:
                                print(f'Error on strat.handelTrade: {e}')    
                            self.handleTrade(d,context)  
                            #print('on_Trade')
                        elif d['type'] == qetype.KEY_ON_ORDER_ERROR or d['type'] == qetype.KEY_ON_CANCEL_CONFIRM:
                            print('stratprocess on order error')
                            self.handleOrder(d,context)
                        elif d['type'] == qetype.KEY_TIMER_PROCESS:
                            #logger.info(f'strat minu report:{self.info_time}')
                            if self.evalmode and strat.datamode == 'minute':
                                try:
                                    savedInsts = strat.instid.copy();
                                    context.curtime = datetime.now()
                                    tday = self.getLocalTradingDay()
                                    self.tradingday = tday.strftime('%Y%m%d')
                                    context.updateTradingday(str(self.tradingday))
                                    if not getTradingDaySaved():
                                        saveTradingDay(self.user, self.token, str(self.tradingday))
                                        setTradingDaySaved(True)
                                    
                                    #context.instsett = qeInstSett()
                                    for inst in savedInsts:
                                        tmpd ={'instid':inst}
                                        #self.autoOrders(tmpd, context)
                                        #self.algoTrade(tmpd,context)
                                    renew = False
                                    if self.wait_all and self.info_time != g_userinfo.info_time_all:
                                        self.info_time = g_userinfo.info_time_all
                                        renew = True
                                    elif not self.wait_all and  self.info_time != g_userinfo.info_time_new : 
                                        self.info_time = g_userinfo.info_time_new 
                                        renew = True
                                    if renew:
                                        bartime = getBarTime(self.info_time, strat.freq)
                                        if bartime:
                                            context.bartime = bartime
                                            strat.onBar(context)
                                    if savedInsts !=  strat.instid:
                                        print("instid changed from ", savedInsts, 'to', strat.instid)
                                        self.changeInstIDs(context, savedInsts, strat.instid)
                                        context.instid = strat.instid
                                except Exception as e:
                                    logger.error(f'Error on strat.handleData {e}',exc_info=True)

                            context.bCrossDay = self.checkCrossDay()
                            if context.bCrossDay:
                                logger.info(f'strat.crossDay {strat.name}')
                               #self.saveTradingday = False
                                savedInsts = strat.instid.copy();
                                try:
                                    context.dayStatistics()
                                    context.crossDay()
                                    
                                    self.crossDay(context)
                                    strat.crossDay(context)
                                except Exception as e:
                                    logger.error(f'Error on strat.crossDay {e}',exc_info=True)
                                if isinstance(strat.instid, str):
                                    strat.instid = [strat.instid]
                                if savedInsts !=  strat.instid:
                                    print("instid changed from ", savedInsts, 'to', strat.instid)
                                    self.changeInstIDs(context, savedInsts, strat.instid)
                                    context.instid = strat.instid
                    except Exception as e:
                        logger.error(f"qestratprocess stratqueue error {e}",exc_info=True )  
                time.sleep(0.001)

    def changeInstIDs(self, context, old, new):
        if context.stratName != 'csvorders':
            if context.runmode == 'simu':
                stgsett = loadSettingData(self.user, self.token)
                stgsett[self.name] = new
                saveSettingDataToDB(self.user, self.token,  stgsett)
            elif context.runmode == 'real':
                stgsett = loadSettingDatareal(self.user, self.token)
                stgsett[self.name] = new
                saveSettingDatarealToDB(self.user, self.token, stgsett)
        
        
        newinsts = []
        for inst in new:
            if not inst in old :
                newinsts.append(inst)
        context.addNewInsts(newinsts)

        #adds = []; removes = [];
        ###need fix
        if 'ctp' in self.mduser:
                changeCtpInstIDs(self.name, new)
        elif 'stock' in self.mduser:
                changeStockInstIDs(self.name, new)
        #if 'sopt' in self.mduser :
        #        changeSoptInstIDs(self.name, new)

           
    def callTimer(self):     
        timer = Timer(self.Timer_interval,self.callTimer)
        d = {}
        d['type'] = qetype.KEY_TIMER_PROCESS
#         logger.info('timer '+str(datetime.now()))
        
        if self.stratQueue:
            self.stratQueue.put(d)
        else:
            logger.info('Timer is pending')
        timer.start()
        return    
  
                
    def confirm(self,code,context):
        # print('confirm')
        #saveRequestToDB(self.user,self.name,-code,context.realTrade)
        if abs(code) == 1:
            print('stop strategy '+str(self.name))
        elif abs(code) == 2:
            print('resume strategy '+str(self.name))
        elif abs(code) == 3:
            print('liquidate strategy '+str(self.name))
    
            
       
    def getPreviousTradingDay(self):
        prev_date = datetime.strptime(self.tradingday,'%Y%m%d') 
        days = 3 if prev_date.weekday() == 0 else 1
        prev_date -= timedelta(days=days)
        while not is_trade_day(prev_date):
            prev_date -= timedelta(days=1)
        return prev_date    
    
    
    def updateTime(self,d,context):
        try:
            self.bDataRead = True
            context.timedigit = d['data']['timedigit']
            self.tradingday = d['data']['tradingday']
            context.updateTradingday(str(self.tradingday))
            if not getTradingDaySaved():
                saveTradingDay(self.user, self.token, str(self.tradingday))
                setTradingDaySaved(True)
            
            curtime = datetime.strptime(d['data']['time'],'%Y%m%d %H:%M:%S.%f')
            if context.curtime is None or curtime > context.curtime :
                context.lasttime=context.curtime
                context.curtime = curtime
            context.curday= context.curtime.strftime('%Y-%m-%d')
            if context.lasttime:
                context.lastday=context.lasttime.strftime('%Y-%m-%d')
            
            if self.recmode and not self.posLoaded:
                self.posLoaded = True
                pos,instPnl = readStratPosition_wrap(self.name, context.tradingday)
                if pos == {} and instPnl == {}:
                    prev_date = self.getPreviousTradingDay() 
                    context.position, context.instClosePnl = readStratPosition_wrap(self.name, prev_date.strftime('%Y%m%d'))
                    
                else:
                    context.position = pos
                    context.instClosePnl = instPnl
                simuaccount.updateStratPostion(context.position)
                context.prodMaxMarg,context.prodTurnover = readStratStat_wrap(self.name,context.tradingday)
                ## write back to database
                writeStratPosition_wrap(self.name, context.tradingday,context.position,context.instClosePnl)
                writeContractTable_wrap(self.name, context.tradingday,context.position)
                writeStratStat_wrap(self.name,context.tradingday, context.prodMaxMarg, context.prodTurnover)
                #context.instsett = qeInstSett()        
            if self.hedgemodel:

                if len(context.dataslide) == len(context.instid):
                    self.hedgemodel_price = self.calculatemul(context)
                    if self.hedgemodel_time == 0:             
                        self.hedgemodel_time = context.timedigit
                    elif context.timedigit > self.hedgemodel_time:
                        hd ={}
                        hd['current'] = self.hedgemodel_price
                        hd['time'] = self.hedgemodel_time
                        #if self.realTrade:
                        #    saveHedgeMarketrealToDB(self.user,self.name, self.instid_hedge, self.hedgemodel_time, d) 
                        #else:    
                        saveHedgeMarketToDB(self.user,self.name, self.instid_hedge, self.hedgemodel_time, hd)
                        self.hedgemodel_time = context.timedigit
        except Exception as e:
            logger.info(f"qestratprocess stratqueue error {e}",exc_info=True )  
 
    def updateData(self,d,context):
        try:
            tempinstid = d['instid']
            context.dataslide[tempinstid] = d['data']
            #print(tempinstid, context.dataslide[tempinstid]['a1_p'])
               
            context.current[tempinstid] = d['data']['current']
        
            if tempinstid in context.lastvol:
                if context.lastvol[tempinstid] == 0:
                    context.lastvol[tempinstid] = context.dataslide[tempinstid]['volume']
                    context.curvol[tempinstid] = 0
                else:
                    context.curvol[tempinstid] = context.dataslide[tempinstid]['volume'] - context.lastvol[tempinstid]
                    context.lastvol[tempinstid] = context.dataslide[tempinstid]['volume']
            else:
                context.curvol[tempinstid] = context.dataslide[tempinstid]['volume'] 
                context.lastvol[tempinstid] = context.dataslide[tempinstid]['volume']
           
                                       
        except Exception as e:
            logger.info(f"qestratprocess stratqueue error {e}",exc_info=True )  

    def calculatemul(self,context):
        a = [0, 0, 0, 0, 0]
        for i in range(len(context.instid)):
            a[i] = context.dataslide[context.instid[i]]['current']
        # print(context.formula)
        multiresult = eval(context.formula, {'a': a[0], 'b': a[1], 'c': a[2], 'd': a[3], 'e': a[4]})
        # print(multiresult)
        return multiresult

    #def refresh(self,d,context):
        #if d['data']['timedigit'] - self.time_delay > 2000:
        #    self.time_delay = d['data']['timedigit']  
        #    if self.realTrade:
        #        #saveStrategyContextrealToDB(self.user,self.name,d['data']['tradingday'],context)
        #        saveStrategyfreqrealToDB(self.user,self.token, self.name,d['data']['tradingday'],context)
        #    else:
        #        #saveStrategyContextToDB(self.user,self.name,d['data']['tradingday'],context)
        #        saveStrategyfreqToDB(self.user,self.token, self.name,d['data']['tradingday'],context)
#       #      print('redis context')        
    
    def crossDay(self,context):
        self.info_time = 0
        #self.info_time_all = 0
        #self.info_time_new = 0
        self.daily_called = False
        logger.info('Strat Process crossDay')        
        if self.recmode:
            lastday = context.lastday.replace('-','')
            writeStratPosition_wrap(self.name, lastday,context.position,context.instClosePnl)
            writeContractTable_wrap(self.name, lastday,context.position)
            writeStratStat_wrap(self.name,lastday, context.prodMaxMarg, context.prodTurnover)
           
    def handleOrder(self,order,context):  
        try:     
            corder = context.orders.get(order['orderid'],None)
            #print('stratprocess ',order)
            if corder:
                #time = str(order['timedigit'])
                corder['status'] = order['status']
                corder['tradevol'] = order['tradevol']
                corder['cancelvol'] = order['cancelvol']
                corder['leftvol'] = order['leftvol']
                corder['stratName'] = self.name
                #corder['time'] = time[:8]+' '+time[8:10]+':'+time[10:12]+':'+time[12:14]+"."+time[14:]
                corder['timedigit'] = order['timedigit']
                corder['errorid'] = order.get('errorid',0)
                corder['errormsg'] = order.get('errormsg','')
                #print('123',order)
                #corder['torderid']=order['torderid']
                '''
                if self.current_timedigit == 0:
                    self.current_timedigit = (order['timedigit'])
                else:
                    self.current_timedigit = max(self.current_timedigit+5,(order['timedigit'])) # increment by 5 ms to identify time change
                
                # logger.debug(str(self.name)+'-'+str(order['timedigit']))
                '''
                context.orders[order['orderid']] = corder
                
                
        except Exception as e:
            logger.error(f"handleorder error {e}",exc_info=True)  


    def handleTrade(self,trade,context):
        try:
            context.simuTrade(trade)
            context.updateData() 
            if self.recmode:
                writeStratPosition_wrap(self.name, context.tradingday,context.position,context.instClosePnl)
                writeContractTable_wrap(self.name, context.tradingday,context.position)
                writeStratStat_wrap(self.name,context.tradingday, context.prodMaxMarg, context.prodTurnover)
                writeStratTrade_wrap(self.name,context.tradingday, trade)
                writeContract_messages_wrap(self.name,context.tradingday, trade)
        except Exception as e:
            logger.error(f"handletrade error {e}",exc_info=True )
    
    
if __name__ == '__main__':
    from multiprocessing import Queue
    class stratBase:
        def initStrat(self, context):
            print("Init strategy which will do nothing.")      

        def handleData(self, context):     
            print("Process time:",context.curtime)

        def crossDay(self,context):
            print('Cross day:', context.lastday)    
    strat = stratBase()
    tq = Queue()
    mq = Queue()
    p = qeStratProcess('n',tq,mq)
    p.ID = 100000
    #p.instid_ex,p.exID = transInstIDs('AG2206.SFE')
    p.stratProcess(strat)


# -*- coding: utf-8 -*-
"""
Created on Wed Oct 13 17:35:49 2021

@author: ScottStation
"""
from datetime import datetime, timedelta
import time
import json
#import os
from .qetype import qetype
import copy
import threading
from .qelogger import logger
from .qecontext import transInstID2Real,transInstID2Context
##from qesimtrader import marketReadyNotice
from .qeredisdb import saveMarketToDB, resetMarketData
from .qestatistics import getPrevTradingDay
from .qeglobal import getInstTraderQueue, getClassInstIDs, getAccidTraderQueue
from .qeinterface import simuqueue
from qesdk import get_valid_instID
from .qeglobal import g_dataSlide
'''
以下为需要订阅行情的合约号，注意选择有效合约；
有效连上但没有行情可能是过期合约或者不再交易时间内导致
'''

import typing
from typing import Optional

from ctpwrapper.ApiStructure import (FensUserInfoField, UserLogoutField,
                                     ReqUserLoginField, QryMulticastInstrumentField)
from ctpwrapper.MdApi import MdApiWrapper


str2bytes = lambda x:bytes(x,'utf-8')

#tqueue = None
mduserspi = None
timer = None
g_mode_724=False

def getLocalTradingDay(now):
    if now.hour < 19:
        return (now.strftime('%Y%m%d'))
    else:
        wday = now.weekday()
        days = 1 if wday != 4 else 3
        return ((now + timedelta(days=days)).strftime('%Y%m%d'))

def getValidPrice(price):
    return 0 if price > 100000000 else price
        


# def timer_callback():
#     global timer, mduserspi
#     logger.debug('timer callback')
#     now = datetime.now()
#     if checkMarketTime(now) :
#         if mduserspi and not mduserspi.bFirstLogin and not mduserspi.connected:
#             mduserspi.login()    
#     timer = threading.Timer(30, timer_callback)
#     timer.start()
        
   
    
def checkMarketTime(now):
    global g_mode_724
    if g_mode_724:
        return True
    curminu = now.hour *100 + now.minute    
    if (curminu > 230 and curminu < 850) or (curminu >= 1530 and curminu <2050)  :
        return False
    elif now.weekday ==6 or (now.weekday==5 and curminu > 240) or (now.weekday == 0 and curminu < 850):
        return False
    return True 
  
def getMarketReady():
    global mduserspi
    
    if mduserspi:
        logger.info(mduserspi.bReady)
        return mduserspi.bReady
    else:
        logger.info('no mduserspi')
        return False

def changeCtpInstIDs(stratName, instIDs):
    global mduserspi
    return mduserspi.changeInstIDs(stratName, instIDs)

def marketLogout():
    global mduserspi
    mduserspi.logout()



# def md_register_strategy(stratName, strat):    
#     global stratGroup, mduserspi, curInstSet ,instExIDs, subID  
#     now = datetime.now()
#     logger.info(f'register {stratName}')
#     if not stratName in stratGroup:
#         stratGroup[stratName] = strat
#         curInstSet.update(strat.instid)
#         instIDs = []
#         for i in range(len(strat.instid_ex)):
#             inst = strat.instid_ex[i]
# #             print(inst)
#             if not inst in subID:
#                 subID.append(inst)
#                 instIDs.append(inst)
#             if not inst in instExIDs:
#                 instExIDs[inst] = strat.exID[i]
#         if len(instIDs) > 0:
#             before_time = getPrevTradingDay(now) + '160000000'    
#             for inst in instIDs:
#                 resetMarketData(inst, before_time)
#             if mduserspi:
#                 mduserspi.SubscribeMarketData(instIDs)
#                 print("subscribe ", instIDs)
#             else:
#                 logger.info('mduserspi is not ready')
            
#             #time.sleep(1)    


       

# def on_data_callback(ctime, instid, mdata):
#     global stratGroup
#     datastr = json.dumps(mdata)
#     for key in stratGroup.keys():
#         if instid in stratGroup[key][0]:
            
#             stratGroup[key][1].put('mdata$$'+instid+'$$'+datastr)
#     print('callback', instid, time)
#     saveMarketToDB(instid, ctime, datastr )
    
class CFtdcMdSpi(MdApiWrapper):
    def Create(self, pszFlowPath: Optional[str] = "",
               bIsUsingUdp: Optional[bool] = False,
               bIsMulticast: Optional[bool] = False) -> None:
        """
        创建MdApi
        :param pszFlowPath: 存贮订阅信息文件的目录，默认为当前目录
        :param bIsUsingUdp:
        :param bIsMulticast:
        """
        super(CFtdcMdSpi, self).Create(pszFlowPath.encode(), bIsUsingUdp, bIsMulticast)

    def __init__(self)->None:
        #self.tapi=tapi
        self.recmode = False
        self.bReady = False
        self.broker = ''
        self.address = ''
        self.connected = False
        self.strats = None
        self.runmode ='simu'
        self.subID=[] #["au2112",","rb2201","TA201"]
        self.curInstSet = set()
        self.instExIDs = {}
        self.instTime = {}
        self.tradingDay = ''
        self.stratInstances = {}
        self.bFirstLogin = True

    def Init(self) -> None:
        """
        初始化运行环境,只有调用后,接口才开始工作
        """
        super(CFtdcMdSpi, self).Init()
        time.sleep(1)  # wait for c++ init

    def Join(self) -> int:
        """
        等待接口线程结束运行
        @return 线程退出代码
        """
        return super(CFtdcMdSpi, self).Join()
        
    def initCallback(self, user, passwd):
        #self.callback = md_callback
        self.user = bytes(user,'utf-8')
        self.passwd = bytes(passwd,'utf-8')
        

    def ReqUserLogin(self, pReqUserLogin: "ReqUserLoginField", nRequestID: int) -> int:
        """
        用户登录请求
        """
        return super(CFtdcMdSpi, self).ReqUserLogin(pReqUserLogin, nRequestID)

    def ReqUserLogout(self, pUserLogout: "UserLogoutField", nRequestID: int) -> int:
        """
         登出请求
        """
        return super(CFtdcMdSpi, self).ReqUserLogout(pUserLogout, nRequestID)

   
    def OnFrontConnected(self) -> None:
        print(u"ctp行情服务器连接成功")
        logger.info (u"ctp行情服务器连接成功")
        now = datetime.now()
        if checkMarketTime(now):
            self.login()
        
        
    def login(self):    
        loginfield = ReqUserLoginField()
        loginfield.BrokerID=self.broker
        loginfield.UserID=self.user
        loginfield.Password=self.passwd
        loginfield.UserProductInfo=str2bytes("qe-pyctp") #"python dll"
        self.connected = True
        self.ReqUserLogin(loginfield,0)
        
    def logout(self):
        logoutfield = UserLogoutField()
        logoutfield.BrokerID = self.broker
        logoutfield.UserID = self.user
        self.ReqUserLogout(logoutfield,0)
    def ReqQryMulticastInstrument(self, pQryMulticastInstrument: "QryMulticastInstrumentField", nRequestID: int) -> int:
        """
        请求查询组播合约
        """
        return super(CFtdcMdSpi, self).ReqQryMulticastInstrument(pQryMulticastInstrument, nRequestID)

    def GetTradingDay(self) -> str:
        """
        获取当前交易日
        @retrun 获取到的交易日
        @remark 只有登录成功后,才能得到正确的交易日
        :return:
        """
        day = super(CFtdcMdSpi, self).GetTradingDay()
        return day.decode()
    
    def OnFrontDisconnected(self, nReason) -> None:
        if self.connected:
            print(u"ctp行情服务器已经断开")
            logger.info(u"ctp行情服务器已经断开")
        self.connected = False    

    def OnHeartBeatWarning(self, nTimeLapse) -> None:
        """
        心跳超时警告。当长时间未收到报文时，该方法被调用。

        :param nTimeLapse: 距离上次接收报文的时间
        :return:
        """
        pass
        
    def OnRspUserLogin(self, pRspUserLogin, pRspInfo, nRequestID, bIsLast) -> None:
        #global tradingDay, instTime
        if pRspInfo.ErrorID == 0:
            logger.info (f"Market Data System Login, SessionID={pRspUserLogin.SessionID},ErrorID={pRspInfo.ErrorID},ErrorMsg={pRspInfo.ErrorMsg}")
        else:
            logger.error (f"Market Data System Login, SessionID={pRspUserLogin.SessionID},ErrorID={pRspInfo.ErrorID},ErrorMsg={pRspInfo.ErrorMsg}")
            
        self.bReady = True
        self.bFirstLogin = False
        #now = datetime.now()
        if pRspUserLogin.TradingDay != '':
            if self.tradingDay != pRspUserLogin.TradingDay:
                if self.tradingDay != '':
                    self.instTime ={}
                self.tradingDay = pRspUserLogin.TradingDay
        elif self.tradingDay == '':
            self.tradingDay = getLocalTradingDay(now)
        #if checkMarketTime(now):
        if len(self.subID) >0:
            time.sleep(1)
            self.SubscribeMarketData([id.encode('utf-8') for id in self.subID])

    def OnRspUserLogout(self, pUserLogout, pRspInfo, nRequestID, bIsLast) -> None:
        """
        登出请求响应
        :param pUserLogout:
        :param pRspInfo:
        :param nRequestID:
        :param bIsLast:
        :return:
        """
        pass
    def OnRspError(self, pRspInfo, nRequestID, bIsLast) -> None:
        """
        错误应答
        :param pRspInfo:
        :param nRequestID:
        :param bIsLast:
        :return:
        """
        pass

    def RegisterFront(self, pszFrontAddress: str) -> None:
        """
        注册前置机网络地址
        @param pszFrontAddress：前置机网络地址。
        @remark 网络地址的格式为：“protocol:# ipaddress:port”，如：”tcp:# 127.0.0.1:17001”。
        @remark “tcp”代表传输协议，“127.0.0.1”代表服务器地址。”17001”代表服务器端口号。
        """
        super(CFtdcMdSpi, self).RegisterFront(pszFrontAddress.encode())

        
    def SubscribeMarketData(self, pInstrumentID: typing.List[str]) -> int:
        """
         订阅行情。
        @param pInstrumentID 合约ID
        :return: int
        """
        logger.info(f'subscribe{pInstrumentID}')
        ids = [item for item in pInstrumentID]
        return super(CFtdcMdSpi, self).SubscribeMarketData(ids)

    def UnSubscribeMarketData(self, pInstrumentID: typing.List[str]) -> int:
        """
        退订行情。
        @param pInstrumentID 合约ID
        :return: int
        """
        logger.info(f'unsubscribe{pInstrumentID}')
        ids = [item for item in pInstrumentID]

        return super(CFtdcMdSpi, self).UnSubscribeMarketData(ids)
        
       
    def OnRtnDepthMarketData(self, pDepthMarketData) -> None:
        #global tradingDay, instTime
        instid = str(pDepthMarketData.InstrumentID)
        
        
        ctime = int(pDepthMarketData.UpdateTime.replace(':',''))*1000+ pDepthMarketData.UpdateMillisec
        
        ## fix repeat time problem
        inc = 0
        if instid in self.instTime.keys() and ctime == self.instTime[instid]['time']:
            inc =  self.instTime[instid]['inc'] + 1

        self.instTime[instid] = {'time':ctime, 'inc':inc}
        
    
        ## DCE actionDay problem
        actionDay = str(pDepthMarketData.ActionDay)

        exID = self.instExIDs[instid] if instid in self.instExIDs else 'XXX'
        #print(instid, type(pDepthMarketData.ActionDay), actionDay=="", exID)
        if (exID == 'DCE' and ctime >= 210000000) or actionDay == "":
            actionDay = datetime.now().strftime('%Y%m%d')
        
        timedigit = int(actionDay)*1000000000 +(ctime+inc)
        timestr = (actionDay +' '+ pDepthMarketData.UpdateTime+'.'+str(pDepthMarketData.UpdateMillisec + 1000)[1:])
        qeinst = transInstID2Context(instid, exID)
        mddata={'tradingday':self.tradingDay,\
        'time': timestr,\
        'timedigit': timedigit,\
        'inst':qeinst,\
        'current': getValidPrice(pDepthMarketData.LastPrice),\
        'presett': getValidPrice(pDepthMarketData.PreSettlementPrice),\
        'preclose':getValidPrice(pDepthMarketData.PreClosePrice),\
        'open':getValidPrice(pDepthMarketData.OpenPrice),\
        'high':getValidPrice(pDepthMarketData.HighestPrice),\
        'low':getValidPrice(pDepthMarketData.LowestPrice),\
        'volume':pDepthMarketData.Volume,\
        'money':pDepthMarketData.Turnover,\
        'position':pDepthMarketData.OpenInterest,\
        #'close':pDepthMarketData.ClosePrice,\
        'upperlimit':getValidPrice(pDepthMarketData.UpperLimitPrice),\
        'lowerlimit':getValidPrice(pDepthMarketData.LowerLimitPrice),\
        'b1_p':getValidPrice(pDepthMarketData.BidPrice1),\
        'b1_v':pDepthMarketData.BidVolume1,\
        'a1_p':getValidPrice(pDepthMarketData.AskPrice1),\
        'a1_v':pDepthMarketData.AskVolume1,}
#         if time.minute % 5 == 1:
        d = {}
        d['type'] = qetype.KEY_MARKET_DATA
#         d['time'] = timedigit
        d['instid'] = qeinst
        d['data'] = mddata
        #print('market',d['instid'],mddata['current'])
        g_dataSlide.update([d])
        self.callback(d)
        
    def OnRspSubMarketData(self, pSpecificInstrument, pRspInfo, nRequestID, bIsLast) -> None:
        if pRspInfo.ErrorID == 0:
            logger.info(u"ctp行情订阅成功："+str(pSpecificInstrument.InstrumentID))
            print(u"ctp行情订阅成功："+str(pSpecificInstrument.InstrumentID))
        else:
            logger.error(f"Subscribe Failed id:{pRspInfo.ErrorID},instid:{pSpecificInstrument.InstrumentID},msg:{pRspInfo.ErrorMsg}")
    
    def OnRspUnSubMarketData(self, pSpecificInstrument, pRspInfo, nRequestID, bIsLast) -> None:
        if pRspInfo.ErrorID == 0:
            logger.info(u"ctp行情退订成功："+str(pSpecificInstrument.InstrumentID))
            print(u"ctp行情退订成功："+str(pSpecificInstrument.InstrumentID))
        else:
            logger.error(f"UnSubscribe Failed id:{pRspInfo.ErrorID},instid:{pSpecificInstrument.InstrumentID},msg:{pRspInfo.ErrorMsg}")
    def callback(self, d):
        try:
        
            instSave = False
            cd = copy.copy(d)        
            if d['type'] == qetype.KEY_MARKET_DATA:
                datastr = json.dumps(d['data'])

               
                if self.strats['async'] and self.recmode:
                    self.strats['queue'].put(cd)
                elif self.stratInsts:
                    for key in self.stratInsts:
                        #print(d['instid'], key, stratGroup, stratGroup[key].instid)
                        
                        if key in self.stratInstances and (d['instid'] in self.stratInsts[key] or key =='csvorders'):  
                            #print('ctp', key, d['instid'])
                            strat = self.stratInstances[key]
                            if self.strats['async']:
                                self.strats['queue'].put(cd)
                            else:
                                self.strats[key]['queue'].put(cd)
                            if strat.datamode == 'tick':
                                instSave = True
                        #print(d['instid'])
                        
        
                #print('callback '+ d['instid'])
                if instSave:
                    saveMarketToDB(d['instid'], d['data']['timedigit'], datastr )
                if self.runmode == 'simu':
                    simuqueue.put(cd)
            elif d['type'] == qetype.KEY_ON_CROSS_DAY:
                if self.runmode == 'simu':
                    simuqueue.put(cd)               
                else:
                    queues = getAccidTraderQueue(0) #getInstTraderQueue(d['instid'])
                    for queue in queues:
                        queue.put(cd)
                    #print('put tequeue',d['data']['timedigit'])
                    
            else:
                logger.error('incorrect type given as type is'+str(d['type']))
        except Exception as e:
            logger.error(f"CTP market callback error {e}",exc_info=True )  
            
    def callTimer(self):
        self.timer = threading.Timer(30, self.timerCallback)
        self.timer.start()   

    def timerCallback(self):
        logger.debug('timer callback')
        now = datetime.now()
        #print('timer callback', checkMarketTime(now), self.bFirstLogin, self.connected)
        if checkMarketTime(now) :
            if not self.bFirstLogin and not self.connected:
                self.login()    
        self.callTimer()

    def register(self):
        #global curInstSet ,instExIDs, subID   
        now = datetime.now()
        logger.info('ctp market register')
        
        self.stratInsts = {}
        for name in self.stratInstances:
            self.stratInsts[name]=self.stratInstances[name].instid
        
        
        realInstSet =set()
        realInstSet.update(self.subID)
        for name in self.stratInsts:
            instid = self.stratInsts[name]
            insts = getClassInstIDs(instid, ['future'])
            self.curInstSet.update(insts)
            self.stratInsts[name] = insts
            realinsts, exids = transInstID2Real(insts)
            for i in range(len(realinsts)):
                if not realinsts[i] in self.instExIDs:
                    self.instExIDs[realinsts[i]] = exids[i]
            
            realInstSet.update(realinsts)
        #print('ctp strats', self.stratInsts)
        if len(realInstSet) > len(self.subID):
            instIDs = []
            for rinst in realInstSet:
                if not rinst in self.subID:
                    instIDs.append(rinst)
            #self.SubscribeMarketData(instIDs)
            before_time = getPrevTradingDay(now) + '160000000'    
            for inst in instIDs:
                resetMarketData(inst, before_time)
            self.subID = list(realInstSet)
        
        if self.recmode:
            self.subID = [get_valid_instID('AU9999')[:6].lower(), get_valid_instID('TF9999')[:6], get_valid_instID('IF9999')[:6]]
        #print('ctp sub', self.subID)
            
    def changeInstIDs(self, name, instIDs):
        adds =[]
        removes = []
        #if name in self.strats:
        #    self.strats[name]['instid'] = instIDs
        if name in self.stratInsts:
            instIDs = getClassInstIDs(instIDs, ['future'])
            newSet = set()
            self.stratInsts[name] = instIDs
            #print(instIDs, name, self.stratInsts)
            for key in self.stratInsts:
                newSet.update(self.stratInsts[key])
            for inst in newSet:
                if not inst in self.curInstSet:
                    adds.append(inst)
            for inst in self.curInstSet:
                if not inst in newSet:
                    removes.append(inst)
            self.curInstSet = newSet
            
            unsubs, exids = transInstID2Real(removes)
            for inst in unsubs:
                if inst in self.subID:
                    self.subID.remove(inst)
                if inst in self.instExIDs:
                    del self.instExIDs[inst]
            subs, exids = transInstID2Real(adds)
            for i in range(len(subs)):
                self.subID.append(subs[i])
                self.instExIDs[subs[i]] = exids[i]
            if not self.recmode:
                if len(subs)> 0:
                    self.SubscribeMarketData([iid.encode('utf-8') for iid in subs])
                if len(unsubs) > 0:
                    self.UnSubscribeMarketData([iid.encode('utf-8') for iid in unsubs])
            else:
                self.subID = [get_valid_instID('AU9999')[:6].lower(), get_valid_instID('TF9999')[:6], get_valid_instID('IF9999')[:6]]
            #print('ctp adds',adds,'removes',removes)
            #print('ctp subs',subs,'unsubs',unsubs)
        return (adds, removes)

    def RegisterNameServer(self, pszNsAddress: str) -> None:
        """
        注册名字服务器网络地址
        @param pszNsAddress：名字服务器网络地址。
        @remark 网络地址的格式为：“protocol:# ipaddress:port”，如：”tcp:# 127.0.0.1:12001”。
        @remark “tcp”代表传输协议，“127.0.0.1”代表服务器地址。”12001”代表服务器端口号。
        @remark RegisterNameServer优先于RegisterFront
        """
        super(CFtdcMdSpi, self).RegisterNameServer(pszNsAddress.encode())

    def RegisterFensUserInfo(self, pFensUserInfo: "FensUserInfoField") -> None:
        """
        注册名字服务器用户信息
        @param pFensUserInfo：用户信息。
        """
        super(CFtdcMdSpi, self).RegisterFensUserInfo(pFensUserInfo)


    def SubscribeForQuoteRsp(self, pInstrumentID: typing.List[str]) -> int:
        """
        订阅询价。
        :param pInstrumentID: 合约ID list
        :return: int
        """
        ids = [bytes(item, encoding="utf-8") for item in pInstrumentID]

        return super(CFtdcMdSpi, self).SubscribeForQuoteRsp(ids)

    def UnSubscribeForQuoteRsp(self, pInstrumentID: typing.List[str]) -> int:
        """
        退订询价。
        :param pInstrumentID: 合约ID list
        :return: int
        """
        ids = [bytes(item, encoding="utf-8") for item in pInstrumentID]

        return super(CFtdcMdSpi, self).UnSubscribeForQuoteRsp(ids)


    def OnRspQryMulticastInstrument(self, pMulticastInstrument, pRspInfo, nRequestID, bIsLast) -> None:
        """
        请求查询组播合约响应
        :param pMulticastInstrument:
        :param pRspInfo:
        :param nRequestID:
        :param bIsLast:
        :return:
        """
        pass



    def OnRspSubForQuoteRsp(self, pSpecificInstrument, pRspInfo, nRequestID, bIsLast) -> None:
        """
        订阅询价应答
        :param pSpecificInstrument:
        :param pRspInfo:
        :param nRequestID:
        :param bIsLast:
        :return:
        """
        pass

    def OnRspUnSubForQuoteRsp(self, pSpecificInstrument, pRspInfo, nRequestID, bIsLast) -> None:
        """
        取消订阅询价应答
        :param pSpecificInstrument:
        :param pRspInfo:
        :param nRequestID:
        :param bIsLast:
        :return:
        """
        pass



    def OnRtnForQuoteRsp(self, pForQuoteRsp) -> None:
        """
        询价通知
        :param pForQuoteRsp:
        :return:
        """
        pass

def runQEMarketProcess(user, passwd, strats,runmode, setting_dict, mode_724=False):
    global mduserspi, g_mode_724, timer
    evalmode = False
    if setting_dict is not None and setting_dict['api'] == 'ctptest':
        evalmode = True
        
    mduserspi = CFtdcMdSpi()
    
    mduserspi.Create("./", False , True)  
    #mduserspi.Init()
    
    mduserspi.strats = strats
    if strats['async']:    
        for s in strats['strat']:
            mduserspi.stratInstances [s.name] = s 
            
    else:
        for name in strats:
            if name != 'async':
                mduserspi.stratInstances[name] = strats[name]['strat']
    #mduserspi.recmode = True if strats['async'] and strats['recmode'] else False
    g_mode_724 = mode_724    
    mduserspi.runmode = runmode
    mduserspi.initCallback(user,passwd) 
    mduserspi.register()
    mduserspi.callTimer()

    pass_flag = True
    # timer = threading.Timer(30, timer_callback)
    # timer.start()

    
    if setting_dict is not None:
        mduserspi.initCallback( setting_dict['investorid'],setting_dict['password'])
        mduserspi.broker = bytes(setting_dict['brokerid'],'utf-8')
        #print(setting_dict)
        address = setting_dict.get('mdaddress',-1)
        if address != -1:
            mduserspi.RegisterFront(address)
            #mduserspi.broker = broker
        else:
            print('user setting has error')
            pass_flag = False

    else:
        #mduserapi.RegisterFront("tcp://101.230.209.178:53313")
        '''
        ctptrader.investor = user_setting['investorid']
        ctptrader.password = user_setting['password']
        ctptrader.broker = user_setting['brokerid']
        ctptrader.address = user_setting['mdaddress']   
        '''
        #mduserapi.RegisterFront("tcp://61.183.150.151:41213")
        print(f"CTP API MD version = {mduserspi.GetApiVersion()}")
        mduserspi.RegisterFront("tcp://222.66.192.247:47213")
        mduserspi.broker = bytes('4300','utf-8')

    if pass_flag:
        #mduserspi.RegisterSpi(mduserspi)  
        mduserspi.Init()    
        mduserspi.Join()
   


if __name__ == '__main__':
    runQEMarketProcess('scott','888888',None)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Jan  5 07:34:57 2022

@author: root
"""

#from datetime import datetime, timedelta
import pandas as pd
#import numpy as np
#from .qelogger import initTableLogger, logger,
from .qelogger import getLoggerPath
#from .qeglobal import getPlatform
import os
import platform


def get_qemonitor_log(user,password,mode):
    
    max_line = 5000
#     current_line = 0    
    log_path = getLoggerPath()
    if mode == 'real':  
        filename = f'{log_path}/qelog_real_{user}'           
    elif mode == 'simu':
        filename = f'{log_path}/qelog_{user}'
    if platform.system().lower() == 'windows':
        filename = filename.replace('/','\\')
    #print(filename)
    pd.set_option('max_colwidth',180)
    if os.path.isfile(filename):
        #try:
        #    df = pd.read_csv(filename,sep='/n',header=None,memory_map=True)  
        #    print('readfile')
        #    #LT = len(df)
        #except:
        #    try:
        #        df = pd.read_csv(filename,sep='/n',header=None,memory_map=True,encoding='GB18030')
        #    except:
        #        print('empty')
        #        return pd.DataFrame()
        #    
        #
        #logdata = df[-max_line:]
        try:
            with open(filename,'r') as f:
                data = f.readlines()
            data = data[-max_line:]
            data = [line.replace('\n','') for line in data]
            logdata = pd.DataFrame(index=range(len(data)),data=data,columns=['日志'])
            #logdata = logdata.sort_index(ascending=False)
            #print(logdata[:10])
        except:
            logdata = pd.DataFrame()
        #logdata = logdata[-max_line:]
    else:
        logdata = pd.DataFrame()

    return logdata
            
    



# -*- coding: utf-8 -*-
"""
Created on Wed Mar 30 15:24:34 2022

@author: ScottStation
"""
import threading
from multiprocessing import Queue
from qesdk import  get_instrument_setting, get_risk_control_parameter
import platform
from datetime import datetime 
import chinese_calendar
import collections
from .prodtables import prodTimes,ticksize,volmult

dbconfig = {'ip': 'data.quantease.store','port': 58888, 'local':'127.0.0.1'};
try:
    from .qedbconfig import dbconfig_outside
    dbconfig = {**dbconfig, **dbconfig_outside}
except:
    pass


class qeDataSlide(collections.UserDict):
    def __missing__(self, key):
        key = str(key)
        if not key in self.data:
            return {"current":0,"presett":0,"preclose":0}    
        return self.data[key]
 
    def __contains__(self, key):
        return str(key) in self.data
 
    def __setitem__(self, key, item):
        self.data[str(key)] = item
    
    def __getitem__(self, key):
        key = str(key)
        if not key in self.data:
            return {"current":0,"presett":0,"preclose":0}    
        return self.data[key]
    def update(self, datalist):
        for data in datalist:
            self.data[data['instid']] = data["data"]
g_dataSlide = qeDataSlide()


mutex = threading.Lock()
import collections
class qeInstSett(collections.UserDict):
    def __missing__(self, key):
        key = str(key)
        if not key in self.data:
            cursett = get_instrument_setting(key)
            #print(cursett)
            if cursett is None:
                raise KeyError
            else:    
                self.data[key] = cursett    
                return cursett    
        return self.data[key]
 
    def __contains__(self, key):
        return str(key) in self.data
 
    def __setitem__(self, key, item):
        self.data[str(key)] = item
    
    def __getitem__(self, key):
        key = str(key)
        if not key in self.data:
            cursett = get_instrument_setting(key)
            #print(cursett)
            if cursett is None:
                raise KeyError
            else:    
                self.data[key] = cursett    
                return cursett    
        return self.data[key]
        
## qeglobal.py
instSetts = qeInstSett()

class globalInfo(object):
    def __init__(self):
        self.info_time_all=0
        self.info_time_new=0
        self.first_t={}

g_tradingdaySaved = False

def setTradingDaySaved(flag):
    global g_tradingdaySaved
    mutex.acquire()
    g_tradingdaySaved = flag
    mutex.release()

def getTradingDaySaved():
    return g_tradingdaySaved
        

g_positionLoaded = False
def setPositionLoaded():
    global g_positionLoaded
    g_positionLoaded = True

def getPositionLoaded():
    global g_positionLoaded
    return g_positionLoaded
        

g_userinfo = globalInfo()  



#def getValidInstIDs(instids):
#    return [getValidInstID(inst) for inst in instids]
        
realQueues = {}
mapClassIDs = {}

def getAccidTraderQueue(accid):
    global realQueues
    #classname = getInstClass(instid)
    if not accid in realQueues:
        realQueues[accid] = Queue()
    return realQueues[accid]

def getInstTraderQueue(instid):
    queues = []
    accids  = getInstAccID(instid)
    for acc in accids:
        queues.append(getAccidTraderQueue(acc))
    return queues


def getClassInstIDs(instIDs, classnames):
    insts = []
    for inst in instIDs:
        if getInstClass(inst) in classnames:
            insts.append(inst)
    return insts        


def setClassAccID(classname, accid):
    global mapClassIDs
    if not classname in mapClassIDs:
        mapClassIDs[classname] = [accid]
    else:
        mapClassIDs[classname].append(accid)

def getInstAccID(instid):
    classname = getInstClass(instid)
    return getClassAccID(classname)
    
def getClassAccID(classname):
    global mapClassIDs
    if classname in mapClassIDs:
        return mapClassIDs[classname]
    else:
        return [0]


def getInstClass(instid):
    if instid[-3:] =='SSE':
        if len(instid) == 10:
            return 'stock'
        else:
            return 'stockoption'
    elif instid[-3:] == 'SGE':
        return 'gold'
    else:
        return 'future'
    
import importlib
def import_source(name, path):
     module_file_path = path
     module_name = name
      
     module_spec = importlib.util.spec_from_file_location(module_name,module_file_path)
     module = importlib.util.module_from_spec(module_spec)
     module_spec.loader.exec_module(module)
     return module    
     
def getExemode():
    try:
        from .execonfig import get_exe_mode
        #print('exemode', get_exe_mode())
        return get_exe_mode()
    except:
        return False


def is_trade_day(date):
    try:
        assert isinstance(date, datetime), 'date must be instance of datetime'
        if date.weekday() > 4:
            return False
        else:
            return chinese_calendar.is_workday(date)
    except Exception as e:
        print('is_trade_day Error:',e)
        return date.weekday() <= 4       
        
#def is_future_expired(instid):
#    getym = lambda x : '20'+x[1:3]+'-'+x[3:5] if x[1].isdigit() else '20'+x[2:4]+'-'+x[4:6]
#    ym = getym(instid)
#    curdate = datetime.today().strftime('%Y-%m-%d')
#    curym = curdate[:7]
#    if curym < ym:
#        #print(ym,curym)
#        return False
#    elif curym == ym:
#        df = get_future_details(instid, expire=curym)
#        if not df.empty:
#            #print(df)
#            return curdate >= df['expire'].iloc[0]
#        else:
#            #print(df)
#            return False
#    else:
#        return True
def get_Instrument_ticksize(instid):


    getProd = lambda x: x[0] if x[1].isdigit() else x[:2]
    return ticksize.get(getProd(instid),1)


def get_Instrument_volmult(instid):


    getProd = lambda x: x[0] if x[1].isdigit() else x[:2]
    return volmult.get(getProd(instid),1)


def getInstrumentSetting(instid):
    res = get_instrument_setting(instid, True)
    if res:
        return res
    else:
        return get_instrument_setting(instid, False)
    

def get_riskctl_paras():
    return get_risk_control_parameter()

def getPlatform():
    return platform.system().lower()



def getProdTime(instid):

    getProd = lambda x: x[0] if x[1].isdigit() else x[:2]
    prodid = getProd(instid).upper()
    exid = instid.split('.')[1].upper()
    if exid == 'SGE':
        prodid = 'SGE'
    res = {}
    if prodid in prodTimes.keys():
        pt = prodTimes[prodid]
        if pt['night'][0] == '':
            res['night'] = []
            res['morning'] = []
        elif pt['morning'][0] != '':
            res['night'] = [(pt['night'][0],'2359')]
            res['morning'] = [('0000',pt['morning'][1])]
        else:
            res['night'] = [(pt['night'][0],pt['night'][1])]
            res['morning'] = []
        if exid in ['CCF','SZE','SSE']:
            res['daytime'] = [('0930','1130')]
            res['daytime'] += [('1300', '1515')] if prodid[0] == 'T' else [('1300', '1500')]
        elif exid == 'SGE':
            res['daytime']= [('0900', '1500')]
        else:
            res['daytime'] = [('0900', '1015'),('1030', '1130'),('1330','1500')]
        return res
    else:
        return {}


def getProdOpenSeconds(timedict):
    #pt = getProdTime(instid)
    pt = timedict
    ret = {}
    today = datetime.today()
    for key in pt.keys():
        if len(pt[key]) == 0:
            ret[key] = [0]
            continue
        for tf in pt[key]:
            [start, end] = tf
            start = today.strftime('%Y%m%d') + start+'00'
            end = today.strftime('%Y%m%d') + end+'00'
            diff = (datetime.strptime(end, '%Y%m%d%H%M%S') - datetime.strptime(start, '%Y%m%d%H%M%S')).seconds
            if not key in ret.keys():
                ret[key] = [diff]
            else:
                ret[key].append(diff)
    return ret        
    #print('dataSlide',dataSlide)    


# -*- coding: utf-8 -*-
"""
Created on Thu Feb  2 18:58:25 2023

@author: ScottStation
"""
import pandas as pd

global_stratmarket_exist = False
try:
    from .qestratmarket import readStratCard
    global_stratmarket_exist = True
except:
    pass    

def writeStratPosition_wrap(strat_name, date, position, instClosePnl):
    if global_stratmarket_exist:
        from .qestratmarket import writeStratPosition
        writeStratPosition(strat_name, date, position, instClosePnl)
    else:
        print("Don't support this interface")
    

def readStratPosition_wrap(strat_name, date, prod=None):
    if global_stratmarket_exist:
        from .qestratmarket import readStratPosition
        return readStratPosition(strat_name, date, prod)
    else:
        print("Don't support this interface")
        return {},{}

def save_trades_wrap(stratName,trade, date):
    if global_stratmarket_exist:
        from .qestratmarket import save_trades
        save_trades(stratName, trade, date)
    else:
        print("Don't support this interface")

def writeStratStat_wrap(strat_name,date, posMaxMarg, posTurnover):
    if global_stratmarket_exist:
        from .qestratmarket import writeStratStat
        writeStratStat(strat_name, date,  posMaxMarg, posTurnover)
    else:
        print("Don't support this interface")
    
def writeStratTrade_wrap(strat_name, date, trade):
    if global_stratmarket_exist:
        from .qestratmarket import writeStratTrade
        writeStratTrade(strat_name, date, trade)
    else:
        print("Don't support this interface")

def writeStratPnl_wrap(strat_name,date,prod, pnl):
    if global_stratmarket_exist:
        from .qestratmarket import writeStratPnl
        writeStratPnl(strat_name,date,prod, pnl)
    else:
        print("Don't support this interface")

def readStratStat_wrap(strat_name,date):
    if global_stratmarket_exist:
        from .qestratmarket import readStratStat
        return readStratStat(strat_name,date)
    else:
        print("Don't support this interface")
        return {},{}

def writeContractTable_wrap(stratname, date, position):
    if global_stratmarket_exist:
        from .qestratmarket import writeContractTable
        writeContractTable(stratname, date, position)
    else:
        print("Don't support this interface")
    
def writeContract_messages_wrap(strat_name, date, trade):
    if global_stratmarket_exist:
        from .qestratmarket import writeContract_messages
        writeContract_messages(strat_name, date, trade)
    else:
        print("Don't support this interface")
def update_stratCard_wrap(stratName, indicators, prodMaxmarg):
    if global_stratmarket_exist:
        from .qestratmarket import update_stratCard
        update_stratCard(stratName, indicators, prodMaxmarg)
    else:
        print("Don't support this interface")
def update_stratCard_append_wrap(stratName, initloss):
    if global_stratmarket_exist:
        from .qestratmarket import update_stratCard_append
        update_stratCard_append(stratName, initloss)
    else:
        print("Don't support this interface")
def clearStratTrades_wrap(strat_name, fromdate=None):
    if global_stratmarket_exist:
        from .qestratmarket import clearStratTrades
        clearStratTrades(strat_name, fromdate)
    else:
        print("Don't support this interface")
def clearStratStat_wrap(strat_name, fromdate=None):
    if global_stratmarket_exist:
        from .qestratmarket import clearStratStat
        clearStratStat(strat_name, fromdate)
    else:
        print("Don't support this interface")
def clearStratPosition_wrap(strat_name, fromdate=None):
    if global_stratmarket_exist:
        from .qestratmarket import clearStratPosition
        clearStratPosition(strat_name, fromdate)
    else:
        print("Don't support this interface")
def readFullStratStat_wrap(strat_name, start_date=None, end_date=None):
    if global_stratmarket_exist:
        from .qestratmarket import readFullStratStat
        return readFullStratStat(strat_name, start_date, end_date)
    else:
        print("Don't support this interface")
        return pd.DataFrame()
    
    
    


#import ctypes
import json
import time
#import argparse
from .secitpdk.secitpdk_dict import (JYLB_SALE,JYLB_BUY,SORT_TYPE_AES,DDLX_XJWT,JYLB_JJRG,
    DDLX_SZSB_QECJCX, DDLX_SZSB_SYCX, DDLX_XJWT, DDLX_SHHB_DSFZYJ,DDLX_SHHB_ZYWDSYZXJ,JYLB_JJSG,
    DDLX_SZSB_DSFZYJ, DDLX_SHHB_BFZYJ, DDLX_SZSB_BFZYJ,DDLX_SZSB_ZYWDSYCX,
    SBJG_WAITING,SBJG_SENDING,SBJG_CONFIRM,SBJG_INVALID,DDLX_SHHB_ZYWDSYCX,
    SBJG_FUNDREQ,SBJG_PARTTRADE,SBJG_COMPLETE,SBJG_PTADPWTD,SBJG_WTDFAIL,SBJG_WITHDRAW,SBJG_MANUAL,
    NOTIFY_PUSH_ORDER,NOTIFY_PUSH_MATCH,NOTIFY_PUSH_INVALID,NOTIFY_PUSH_WITHDRAW,JYLB_ETFSG)
from .secitpdk.secitpdk_struct import (ITPDK_BatchOrder)
from .secitpdk.secitpdk import SECITPDK, cont
from .qelogger import logger
import os
import copy
from threading import Timer
from .qestatistics import g_stat
from .qetype import qetype
from .qeglobal import setPositionLoaded
from .qeredisdb import saveTradeDatarealToDB,saveOrderDatarealToDB
Timer_interval = 2

curHtsTrader = None
class HtsTrader(object):
    def __init__(self, user,account,classname,strats,queue,khh):
        self.tqueue = queue
        self.account = account
        self.account.user = user
        self.account.investorid = khh
        self.strats = strats
        self.user = user
        self.classname = classname
        self.is_connected = False
        self.khh = khh
        self.posLoad = False
        self.accLoad = False
        self.accinfoLoad = False
        ### save for today
        self.updatetime = 0
        self.curday = ''
        self.turnover = 0
        self.posDict = {}
        
    def updateTime(self):
        self.sysday = SECITPDK.GetSystemDate()
        self.tradingday = SECITPDK.GetTradeDate()
        self.timestr = SECITPDK.GetReviseTime()
        self.timedigit = SECITPDK.GetReviseTimeAsLong()
        
        
        if self.account.tradingDay == '':
            self.account.loadFromDB(self.tradingday)
        self.account.current_timedigit = self.timedigit
        self.account.setTradingDay ( self.tradingday  )      
        
        if self.tradingday != self.curday:
            if self.curday != '':
                self.crossday()
            self.curday = self.tradingday

        if self.timedigit - self.updatetime > 3:
            self.updatetime = self.timedigit
            self.account.saveToDB()        

    def TraderProcess(self):
        logger.info(u"start Trader") 
        print("start Trader")
        self.updateTime()
        self.qryAccInfo()
        self.qryAccount()
        self.qryPosition()
        self.callTimer()
        if self.tqueue and self.posLoad and self.accLoad and self.accinfoLoad:
            logger.info('tqueue is ready')
            while True:
                try:
                    while not self.tqueue.empty():
#                         print('.')
                        d = self.tqueue.get(block=True, timeout=1)
                        self.process(d)
                    time.sleep(0.001)
                except Exception as e:
                    logger.error(f"ctptrader tqueue error {e}",exc_info=True )  
        else:
            logger.info('tqueue is not ready')
                    
    def process(self, d):
        ##处理queue信息
        if d['type'] == qetype.KEY_SEND_ORDER:
            self.sendOrder(d)
        elif d['type'] == qetype.KEY_CANCEL_ORDER:
            self.cancelOrder(d)
        
        elif d['type'] == qetype.KEY_ON_ORDER:
            self.onOrder(d)
        elif d['type'] == qetype.KEY_ON_TRADE:
            self.onTrade(d)              
        elif d['type'] == qetype.KEY_ON_ORDER_ERROR:
            self.onOrderError(d)
        elif d['type'] == qetype.KEY_ON_CANCEL_CONFIRM:
            self.onCancelConfirm(d)

#         elif d['type'] == KEY_ON_TRADE_ERROR:
#             self.onTradeError(d)
        elif d['type'] == qetype.KEY_TIMER:
            self.getStatistic()
            #self.heartBeat(True)
#         elif d['type'] == KEY_ON_INSTRUMENT:
#             self.onInstrument(d)
#         elif d['type'] == KEY_ON_POSITION_DETAIL:
#             self.onPositionDetail(d)
        else:
            logger.error('incorrect type given as type is '+str(d['type']))


    def sendOrder(self,d):
        ## 下单
        self.entrustOrder(d)
        
    def cancelOrder(self,d):
        ## 撤单
        self.orderWithdraw(d['orderid'])
        
    def onOrder(self,d):
        order = self.account.orders.get(d['orderid'],None)
        if order:
            order['status'] = d['status']

            d['direction'] = order['direction']
            d['action'] = order['action']
            d['closetype'] = 'auto' 
            d['timedigit'] = order['timedigit']
            #time = str(d['timedigit'])
            #d['time'] = time[:8]+' '+time[8:10]+':'+time[10:12]+':'+time[12:14]+"."+time[14:]
            d['time'] = order['time']
            d['stratName'] = order['stratName']
            d['instid'] = order['instid']
            d['type'] = qetype.KEY_ON_ORDER
            if d['status'] in [qetype.KEY_STATUS_CANCEL, qetype.KEY_STATUS_PTPC , qetype.KEY_STATUS_REJECT]:
                order['cancelvol'] = d['volume'] - d['tradevol']
                order['leftvol'] = 0  
            elif d['status'] == qetype.KEY_STATUS_ALL_TRADED :
                order['cancelvol'] = 0
                order['leftvol'] = 0
            else:
                order['cancelvol'] = 0
                order['leftvol'] = d['volume'] - d['tradevol']
            d['cancelvol'] = order['cancelvol']
            d['leftvol'] = order['leftvol']  
            #if d['stratName'] == '':
            #    print('unresolved order', d['orderid'])
            d['accid'] = self.account.accid
            d['timecond'] = order['timecond']
            #d['sessionid'] = order['sessionid']
            #print('errormsg',d.keys())
            saveOrderDatarealToDB(self.account.user,self.account.token, self.account.tradingDay, order )
            self.account.orders[d['orderid']] = order
            self.callback(d)
#             self.account.saveToDB()
        else:
            logger.info('rspOrder orderid is not found '+str(d['orderid']))

    def onTrade(self,d):
        ## 订单成交
        order = self.account.orders.get(d['orderid'],None)
        if order or d['from'] != 'RtnTrade':
            d['action'] = 'open' 
            d['closetype'] = 'auto' 
            d['direction'] = order['direction']
            #d['timedigit'] = int(datetime.now().strftime("%Y%m%d%H%M%S")+'001')
            d['timedigit'] = SECITPDK.GetReviseTimeAsLong()
            d['stratName'] = order['stratName']
            d['instid'] = order['instid']
            d['type'] = qetype.KEY_ON_TRADE

            order['status'] = d['status']
            order['tradevol'] += d['tradevol']
            if d['status'] in [qetype.KEY_STATUS_CANCEL, qetype.KEY_STATUS_PTPC , qetype.KEY_STATUS_REJECT]:
                order['cancelvol'] = d['volume'] - d['tradevol']
                order['leftvol'] = 0  
            elif d['status'] == qetype.KEY_STATUS_ALL_TRADED :
                order['cancelvol'] = 0
                order['leftvol'] = 0
            else:
                order['cancelvol'] = 0
                order['leftvol'] = d['volume'] - d['tradevol']
            self.account.orders[d['orderid']] = order
            saveOrderDatarealToDB(self.account.user,self.account.token, self.account.tradingDay, order)

            trade = {}
            trade['instid'] = d['instid']
            trade['action'] = d['action']
            trade['dir'] = d['direction']
            trade['orderid'] = d['orderid']
            trade['tradevol'] = d['tradevol']
            trade['tradeprice'] = d['tradeprice']
            trade['stratName'] = d['stratName']
            trade['timedigit'] = d['timedigit']
            trade['closetype'] = d['closetype']
            trade['date'] = SECITPDK.GetTradeDate()
            trade['time'] = SECITPDK.GetReviseTime()
            
            tradeid = g_stat.getNewTradeID(int(str(d['timedigit'])[2:]+'00'))
            #while  tradeid in self.account.trades:
            #    tradeid += 1
            trade['tradeid'] = tradeid
            self.account.trades[tradeid] = trade
            #if d['stratName'] == '':
            #    print('unresolved trade', d['orderid'])
            trade['accid'] = self.account.accid
            saveTradeDatarealToDB(self.account.user, self.account.token, self.account.tradingDay, trade )
            self.callback(d)
            self.account.saveToDB()
            dirstr = 'long' if d['dir']>0 else 'short'
            self.account.updateWinLossParas(dirstr,  d['tradeprice'], d['tradevol'], order['closetype'], order['instid'])
        
    def onOrderError(self,d):
        ## 订单失败
        order = self.account.orders.get(d['orderid'],None)
        if order:
            d['stratName'] = order['stratName']
            d['instid'] = order['instid']
            #d['incoming_orderid'] = order['incoming_orderid']      
            d['status'] = qetype.KEY_STATUS_REJECT
            d['direction'] = order['direction']
            d['timedigit'] = order['timedigit']   
            d['tradevol'] = 0    
            d['cancelvol'] = d['volume']
            d['leftvol'] = 0
            ## add keys            
            d['action'] = order['action']
            d['closetype'] = order['closetype']
            d['accid'] = self.account.accid
            d['timecond'] = order['timecond']
            d['time'] = order['time']
            d['type'] = qetype.KEY_ON_ORDER_ERROR
            
            self.callback(d)
            order['errorid'] = d['errorid']
            order['errormsg'] = d['errormsg']
            order['status'] = qetype.KEY_STATUS_REJECT
            order['tradevol']  = 0
            order['cancelvol'] = d['volume']
            order['leftvol'] = 0  
            #print('error errormsg',order.keys(),d.keys())
            saveOrderDatarealToDB(self.account.user,self.account.token, self.account.tradingDay, order )
            self.account.orders[d['orderid']] = order

    def onCancelConfirm(self,d):
        ## 撤单回报
        order = self.account.orders.get(d['orderid'],None)
        if order:
            order['errorid'] = d['errorid']
            order['errormsg'] = d['errormsg']
            order['status'] = d['status']
            if d['status'] == qetype.KEY_STATUS_CANCEL:
                order['cancelvol'] = order['volume'] - order['tradevol']
                order['leftvol'] = 0
            else:
                order['cancelvol'] = 0
                order['leftvol'] = order['volume'] - order['tradevol']
            saveOrderDatarealToDB(self.account.user,self.account.token, self.account.tradingDay, order )
            self.account.orders[d['orderid']] = order    
            d = copy.deepcopy(order)
            d['type'] = qetype.KEY_ON_CANCEL_CONFIRM
            self.callback(d)

    def callback(self, d):
        ## 回调strat接口，返回策略订单信息
        if d['stratName'].replace(' ','') != '':
            if self.strats:
                stratQueue = self.strats.get(d['stratName'],None)
                if stratQueue:
                    stratQueue['queue'].put(d)
                else:
                    logger.error('callback '+str(d['stratName'])+' is not found')

    def callTimer(self):
        ## 定时器
        global  Timer_interval
        timer = Timer(Timer_interval,self.callTimer)
        d = {}
        d['type'] = qetype.KEY_TIMER
#         logger.info('timer '+str(datetime.now()))
        if self.tqueue:
            self.tqueue.put(d)
        else:
            logger.info('Timer is pending')
        timer.start()       
        
    def getStatistic(self):
        ## 获取统计数据
        self.qryAccount()
        self.qryPosition()

        self.updateTime()
        
    def crossDay(self):
        ## 跨天
        self.turnover = 0
        self.posDict = {}
    
    def onAccount(self,d):
        ## 账户回报
        self.account.balance = d['balance']
        self.account.avail = d['available'] 
        self.account.dayfees = d['commission'] 
        self.account.margin = d['margin'] 
        self.account.closeProf = d['closeProfit'] 
        self.account.posProf = d['positionProfit']
        self.account.frozenMarg = d['frozenMarg']
        self.account.daypnl = d['closeProfit'] #+ d['positionProfit']
        self.account.totalpnl = self.account.accupnl + self.account.daypnl
        self.account.totalfees = self.account.accufees + self.account.dayfees
        self.account.maxmarg = max(self.account.margin, self.account.maxmarg)
        
        if not self.accLoad:
            self.accLoad = True
            self.account.saveToDB()
            print('ctp账户资金信息加载完毕',self.account.balance)
            self.account.setLoadReady()        
        
        
    def onPosition(self,d):
        ## 仓位回报
        self.account.position = copy.copy(d['data'])
        self.account.turnover += float(d['turnover'])
        if not self.posLoad:
            self.posLoad = True
            self.account.saveToDB()
            setPositionLoaded()
            print('ctp账户持仓信息加载完毕')
            logger.info('ctp账户持仓信息加载完毕')
            ##self.account.setLoadReady() ##wait for account load ready
            
    #############################
    ## 以下为HTS独有
    def getTradableQty(self, stcode, exid ,orderdir, price, ordertype):
        ## 获取可交易标的
        gdh = self.shgdh if exid == 'SH' else self.szgdh
        kmsl = SECITPDK.TradableQty(self.khh, stcode, exid, orderdir, price, ordertype, gdh)
        if kmsl >= 0:
            print ("Tradeable qty =", kmsl)
            return kmsl
        else:
            retcode, retmsg = SECITPDK.GetLastError()
            cont.error("TradableQty failed, {}: {}".format(retcode, retmsg))
            return 0

    def entrustOrder(self, order):
        try:
            orderdir = JYLB_BUY if order['dir'] > 0  else JYLB_SALE 
            stock = order['instid'].split('.')
            stcode = stock[0]
            exid = stock[1] if len(stock) > 1 else ''
            assert exid in ['SSE',"SZE"], "exchange id error"
            exid = 'SH' if exid == 'SSE' else 'SZ'
            ordertype = order['ordertype']
            timecond = order['timecond']
            if ordertype == 'limit':
                if timecond == 'FOK' :
                    if exid == 'SZ':
                        htsorder = DDLX_SZSB_QECJCX
                    else:
                        raise ValueError('limit FOK order not supported for SH')    
                elif timecond == 'FAK':
                    if exid == 'SZ':
                        htsorder = DDLX_SZSB_SYCX
                    else:
                        raise ValueError('limit FAK order not supported for SH')    
                else:
                    htsorder = DDLX_XJWT

            elif ordertype == 'opponent':
                htsorder = DDLX_SHHB_DSFZYJ if exid == 'SH' else DDLX_SZSB_DSFZYJ
            elif ordertype == 'quote':
                htsorder = DDLX_SHHB_BFZYJ if exid == 'SH' else DDLX_SZSB_BFZYJ
            elif ordertype == 'market':
                if timecond == 'FAK':
                    htsorder = DDLX_SHHB_ZYWDSYCX if exid == 'SH' else DDLX_SZSB_ZYWDSYCX
                elif timecond == 'FOK':
                    raise ValueError('market FOK order not supported')
                elif timecond == 'FAL':
                    if exid == 'SH':
                        htsorder = DDLX_SHHB_ZYWDSYZXJ
                    else:
                        raise ValueError('market FAL order not supported for SZ')          
            
            qty = self.getTradableQty(stcode, exid ,orderdir, order['price'], htsorder)
            if qty == 0:
                msg = '下单失败: 可委托数量为0'
                cont.error(msg)
                logger.error(msg)
                order['errorid'] = -1
                order['errormsg']=msg
                self.onOrderError(order)
            elif order['volume'] > qty:
                msg = '下单失败: 下单数量大于可委托数量'
                cont.error(msg)
                logger.error(msg)
                order['errorid'] = -2
                order['errormsg']=msg
                self.onOrderError(order)
            else:
                gdh = self.shgdh if exid == 'SH' else self.szgdh

                wth = SECITPDK.OrderEntrust(self.khh, exid, stcode, orderdir, order['volume'], order['price'], htsorder, gdh)
                if wth > 0:
                    print ("下单成功, wth=", wth)
                    orderid = wth
                    order['orderid'] = wth
                    order['time'] = SECITPDK.GetReviseTime()
                    order['timedigit'] = SECITPDK.GetReviseTimeAsLong()
                    self.account.orders[orderid] = order

                else:
                    retcode, retmsg = SECITPDK.GetLastError()
                    cont.error("下单失败, {}: {}".format(retcode, retmsg))  
                    order['errorid'] = retcode
                    order['errormsg']=retmsg
                    self.onOrderError(order)
        except Exception as e:
            cont.error('entrustOrder error: '+str(e))
            logger.error('下单失败: '+str(e))

    def qryAccount(self):
        """ 查询资金 """
        khh = self.khh
        print ("=========== queryfundinfo_example ============")
        size, arZjzh = SECITPDK.QueryFundInfo(khh)
        if size < 0:
            retcode, retmsg = SECITPDK.GetLastError()
            cont.error("Query fund info failed, {}: {}".format(retcode, retmsg))
        else:
            print ("Query fund info success, result num:", size)
            d = {}
            d['type'] = qetype.KEY_ON_ACCOUNT
            for i in range(size):
                d['available'] = arZjzh[i].FundAvl
                d['commission'] = 0 ## unsupported
                d['margin'] = arZjzh[i].MarketValue
                d['closeProfit'] = arZjzh[i].DateProfit - arZjzh[i].UnclearProfit
                d['positionProfit'] = arZjzh[i].UnclearProfit
                d['frozenMarg'] = arZjzh[i].FrozenBalance
                d['balance'] = arZjzh[i].TotalAsset
                d['deposit'] = 0 ## unsupported
                d['withdraw'] = 0 ## unsupported               
                print ("AccountId: {}, FundAccount: {}, MoneyType:{}, OrgCode:{}, MasterFlag:{}, AccountType:{}, LastBalance:{}, CurrentBalance: {}, FrozenBalance:{}, FundAvl: {}, TotalAsset:{}, MarketValue:{}, UncomeBalance:{}, FetchBalance:{}, UpdateTime:{}, SettleBalance:{}".format(arZjzh[i].AccountId, arZjzh[i].FundAccount, arZjzh[i].MoneyType, arZjzh[i].OrgCode, arZjzh[i].MasterFlag, arZjzh[i].AccountType, arZjzh[i].LastBalance, arZjzh[i].CurrentBalance, arZjzh[i].FrozenBalance, arZjzh[i].FundAvl, arZjzh[i].TotalAsset, arZjzh[i].MarketValue, arZjzh[i].UncomeBalance, arZjzh[i].FetchBalance, arZjzh[i].UpdateTime, arZjzh[i].SettleBalance))
                break
            self.onAccount(d)

    def qryPosition(self):    
        """ 查询持仓 """
        #print ("=========== queryzqgl_example ============")
        khh =self.khh
        size, arZQGL = SECITPDK.QueryPositions(khh, SORT_TYPE_AES, 0, 0, "", "", "", 1)
        if size < 0:
            retcode, retmsg = SECITPDK.GetLastError()
            cont.error("查询持仓失败, {}: {}".format(retcode, retmsg))
        else:
            print ("Query zqgl success, result num:", size)

            self.posDict = {}
            self.turnover = 0
            for i in range(size):
                pos = {'long': {'volume': 0, 'poscost': 0, 'yesvol': 0},'short': {'volume': 0, 'poscost': 0, 'yesvol': 0}}
                inst = arZQGL[i].StockCode
                pos['long']['volume'] = arZQGL[i].CurrentQty + arZQGL[i].PreQty
                pos['long']['poscost'] = arZQGL[i].CostPrice
                pos['long']['yesvol'] = arZQGL[i].PreQty
                self.posDict[inst] = pos
                self.turnover += arZQGL[i].RealBuyBalance + arZQGL[i].RealSellBalance
                #print ("AccountId: {}, SecuAccount: {}, StockCode: {}, CurrentQty: {}, MarketValue: {}, CostPrice: {}, DiluteCostPrice: {}, KeepCostPrice: {}, QtyAvl: {}".format(arZQGL[i].AccountId, arZQGL[i].SecuAccount, arZQGL[i].StockCode, arZQGL[i].CurrentQty, arZQGL[i].MarketValue, arZQGL[i].CostPrice, arZQGL[i].DiluteCostPrice, arZQGL[i].KeepCostPrice, arZQGL[i].QtyAvl))
        d = {}
        d['type'] = qetype.KEY_ON_POSITION
        d['data'] = copy.copy(self.posDict)
        d['turnover'] = self.turnover
        self.onPosition(d)

    def qryAccInfo(self):
        ## 查询交易所账户信息
        szgdh = ""
        shgdh = ""
        print ("=========== queryaccinfo_example ============")
        size, arGdh = SECITPDK.QueryAccInfo(self.khh)
        if size < 0:
            retcode, retmsg = SECITPDK.GetLastError()
            cont.error("Query account info failed, {}: {}".format(retcode, retmsg))
        else:
            print ("Query account info success, result num:", size)
            for i in range(size):
                print ("AccountId: {}, Market: {}, SecuAccount: {}, HolderName:{}, FundAccount: {}, OrgCode:{}, MoneyType:{}, TradeAccess: {}, HolderType:{}".format(arGdh[i].AccountId, arGdh[i].Market, arGdh[i].SecuAccount, arGdh[i].HolderName, arGdh[i].FundAccount, arGdh[i].OrgCode, arGdh[i].MoneyType, arGdh[i].TradeAccess, arGdh[i].HolderType))

                if "SH" == arGdh[i].Market:
                    shgdh = arGdh[i].SecuAccount
                if "SZ" == arGdh[i].Market:
                    szgdh = arGdh[i].SecuAccount
            self.accinfoLoad = True
            self.szgdh = szgdh
            self.shgdh = shgdh        
    def orderWithdraw(self, orderid):
        cdwth = SECITPDK.OrderWithdraw(self.khh, "SZ", orderid)
        if cdwth > 0:
            print ("撤单成功, 订单号=", cdwth)
        else:
            retcode, retmsg = SECITPDK.GetLastError()
            cont.error("撤单失败, {}: {}".format(retcode, retmsg))
            logger.error("撤单失败, {}: {}".format(retcode, retmsg))

        
    def qryOrder(self):
        pass
    
    def qryTrade(self):
        pass
    
    def processStatus(self,statusstr):
        mapdict ={SBJG_WAITING:qetype.KEY_STATUS_UNKNOWN,
                    SBJG_SENDING:qetype.KEY_STATUS_UNKNOWN,
                    SBJG_CONFIRM:qetype.KEY_STATUS_PENDING,
                    SBJG_INVALID:qetype.KEY_STATUS_REJECTED,
                    SBJG_FUNDREQ:qetype.KEY_STATUS_PENDING,
                    SBJG_PARTTRADE:qetype.KEY_STATUS_PART_TRADED,
                    SBJG_COMPLETE:qetype.KEY_STATUS_ALL_TRADED,
                    SBJG_PTADPWTD:qetype.KEY_STATUS_PTPC,
                    SBJG_WITHDRAW:qetype.KEY_STATUS_CANCELLED,
                    SBJG_WTDFAIL:qetype.KEY_STATUS_CANCELL_FAILED,
                    SBJG_MANUAL:qetype.KEY_STATUS_UNKNOWN}
        return mapdict[statusstr]

    def on_msg_call_back(self,stime, smsg, ntype):
        """ 交易所确认、废单、成交消息推送回调 """
        msg = smsg.decode("gb2312", "strict")
        data = json.loads(msg)
        d = {}
        if NOTIFY_PUSH_ORDER == ntype:
            d['type'] = qetype.KEY_ON_ORDER
            d['orderid'] = data["WTH"]
            d['status'] = self.processStatus(data["SBJG"])
            d['errormsg'] = data['JGSM']
            print ("Receive order confrim msg -- KFSBDBH: {}, WTH: {}, SBJG:{}, JGSM:{}".format(data["KFSBDBH"], data["WTH"], data["SBJG"], data["JGSM"]))
        if NOTIFY_PUSH_WITHDRAW == ntype:
            d['type'] = qetype.KEY_ON_CANCEL_CONFIRM
            d['orderid'] = data["WTH"]
            d['status'] = self.proceeStatus(data["SBJG"])
            d['errormsg'] = data['JGSM'].split(':')[1]
            d['errorid'] = data['JGSM'].split(':')[0]
            print ("Receive withdraw confirm msg -- KFSBDBH: {}, WTH: {}, SBJG:{}, JGSM:{}".format(data["KFSBDBH"], data["WTH"], data["SBJG"], data["JGSM"]))
        if NOTIFY_PUSH_MATCH == ntype:
            d['type'] = qetype.KEY_ON_TRADE
            d['orderid'] = data["WTH"]
            d['status'] = self.processStatus(data["SBJG"])
            d['tradeprice'] = data["CJJG"]
            d['tradevol'] = data["CJSL"]
            d['errormsg'] = data['JGSM']
            print ("Receive order report msg -- KFSBDBH: {}, WTH: {}, CJJG: {}, CJSL: {}, SBJG:{}, JGSM:{}".format(data["KFSBDBH"], data["WTH"], data["CJJG"], data["CJSL"], data["SBJG"], data["JGSM"]))
        if NOTIFY_PUSH_INVALID == ntype:
            d['type'] = qetype.KEY_ON_ORDER_ERROR
            d['orderid'] = data["WTH"]
            d['status'] = self.processStatus(data["SBJG"])
            d['errormsg'] = data['JGSM'].split(':')[1]
            d['errorid'] = data['JGSM'].split(':')[0]
            print ("Receive order failed msg -- KFSBDBH: {}, WTH: {}, SBJG:{}, JGSM:{}".format(data["KFSBDBH"], data["WTH"], data["SBJG"], data["JGSM"]))
        if self.tqueue:
            self.tqueue.put(d)


    def on_connevent_call_back(self, skhh, sconnkey, nevent, data):
        """ 服务连接事件回调 """
        print ("Receive conn evetn -- KHH: {}, Key: {}, Event: {}".format(skhh, sconnkey, nevent))
        
    def on_func_call_back(self,stime, smsg, ntype):
        """ 异步接口回调 """
        msg = smsg.decode("gb2312", "strict")
        data = json.loads(msg)
        print ("Async func call back -- KFSBDBH: {}, RETCODE: {}, RETNOTE:{}".format(data["KFSBDBH"], data["RETCODE"], data["RETNOTE"]))


def on_func_call_back(stime, smsg, ntype):
    """ 异步接口回调 """
    # print "on_func_call_back"
    msg = smsg.decode("gb2312", "strict")
    data = json.loads(msg)
    print ("Async func call back -- KFSBDBH: {}, RETCODE: {}, RETNOTE:{}".format(data["KFSBDBH"], data["RETCODE"], data["RETNOTE"]))


def on_msg_call_back(stime, smsg, ntype):
    """ 交易所确认、废单、成交消息推送回调 """
    msg = smsg.decode("gb2312", "strict")
    data = json.loads(msg)

    if NOTIFY_PUSH_ORDER == ntype:
        print ("Receive order confrim msg -- KFSBDBH: {}, WTH: {}, SBJG:{}, JGSM:{}".format(data["KFSBDBH"], data["WTH"], data["SBJG"], data["JGSM"]))
    if NOTIFY_PUSH_WITHDRAW == ntype:
        print ("Receive withdraw confirm msg -- KFSBDBH: {}, WTH: {}, SBJG:{}, JGSM:{}".format(data["KFSBDBH"], data["WTH"], data["SBJG"], data["JGSM"]))
    if NOTIFY_PUSH_MATCH == ntype:
        print ("Receive order report msg -- KFSBDBH: {}, WTH: {}, CJJG: {}, CJSL: {}, SBJG:{}, JGSM:{}".format(data["KFSBDBH"], data["WTH"], data["CJJG"], data["CJSL"], data["SBJG"], data["JGSM"]))
    if NOTIFY_PUSH_INVALID == ntype:
        print ("Receive order failed msg -- KFSBDBH: {}, WTH: {}, SBJG:{}, JGSM:{}".format(data["KFSBDBH"], data["WTH"], data["SBJG"], data["JGSM"]))


def on_connevent_call_back(skhh, sconnkey, nevent, data):
    """ 服务连接事件回调 """
    print ("Receive conn evetn -- KHH: {}, Key: {}, Event: {}".format(skhh, sconnkey, nevent))


def callback_example():
    SECITPDK.SetFuncCallback(on_func_call_back)
    SECITPDK.SetMsgCallback(on_msg_call_back)
    SECITPDK.SetConnEventCallback(on_connevent_call_back)


def login_example(key, khh, pwd, wtfs):
    """ 登录 """
    print ("=========== login_example ============")
    SECITPDK.SetWTFS(wtfs)
    SECITPDK.SetNode("pysecitpdk")
    token = SECITPDK.TradeLogin(key, khh, pwd)
    if token > 0:
        print ("Login success, token:", token)
    else:
        retcode, retmsg = SECITPDK.GetLastError()
        cont.error("Login error, {}: {}".format(retcode, retmsg))


def time_example():
    """ 获取服务端交易日期、系统日期 """
    print ("=========== time_example ============")
    print ("Server system date:", SECITPDK.GetSystemDate())
    print ("Server trade date:", SECITPDK.GetTradeDate())
    print ("Revise date:", SECITPDK.GetReviseTime())

def querynodeinfo_example(key, khh):
    """ 查询客户节点信息 """
    global szgdh, shgdh
    print ("=========== querynodeinfo_example ============")
    size, arKhjd = SECITPDK.QueryCusNodeinfo(key, khh)
    if size < 0:
        retcode, retmsg = SECITPDK.GetLastError()
        cont.error("Query fund info failed, {}: {}".format(retcode, retmsg))
    else:
        print ("Query fund info success, result num:", size)
        for i in range(size):
            print ("AccountId: {}, SystemType: {}, Market:{}, NodeID: {}".format(arKhjd[i].AccountId, arKhjd[i].SystemType, arKhjd[i].Market, arKhjd[i].NodeID))


def queryfundinfo_example(khh):
    """ 查询资金 """
    print ("=========== queryfundinfo_example ============")
    size, arZjzh = SECITPDK.QueryFundInfo(khh)
    if size < 0:
        retcode, retmsg = SECITPDK.GetLastError()
        cont.error("Query fund info failed, {}: {}".format(retcode, retmsg))
    else:
        print ("Query fund info success, result num:", size)
        for i in range(size):
            print ("AccountId: {}, FundAccount: {}, MoneyType:{}, OrgCode:{}, MasterFlag:{}, AccountType:{}, LastBalance:{}, CurrentBalance: {}, FrozenBalance:{}, FundAvl: {}, TotalAsset:{}, MarketValue:{}, UncomeBalance:{}, FetchBalance:{}, UpdateTime:{}, SettleBalance:{}".format(arZjzh[i].AccountId, arZjzh[i].FundAccount, arZjzh[i].MoneyType, arZjzh[i].OrgCode, arZjzh[i].MasterFlag, arZjzh[i].AccountType, arZjzh[i].LastBalance, arZjzh[i].CurrentBalance, arZjzh[i].FrozenBalance, arZjzh[i].FundAvl, arZjzh[i].TotalAsset, arZjzh[i].MarketValue, arZjzh[i].UncomeBalance, arZjzh[i].FetchBalance, arZjzh[i].UpdateTime, arZjzh[i].SettleBalance))


def queryaccinfo_example(khh):
    """ 查询股东信息 """
    szgdh = ""
    shgdh = ""
    print ("=========== queryaccinfo_example ============")
    size, arGdh = SECITPDK.QueryAccInfo(khh)
    if size < 0:
        retcode, retmsg = SECITPDK.GetLastError()
        cont.error("Query account info failed, {}: {}".format(retcode, retmsg))
    else:
        print ("Query account info success, result num:", size)
        for i in range(size):
            print ("AccountId: {}, Market: {}, SecuAccount: {}, HolderName:{}, FundAccount: {}, OrgCode:{}, MoneyType:{}, TradeAccess: {}, HolderType:{}".format(arGdh[i].AccountId, arGdh[i].Market, arGdh[i].SecuAccount, arGdh[i].HolderName, arGdh[i].FundAccount, arGdh[i].OrgCode, arGdh[i].MoneyType, arGdh[i].TradeAccess, arGdh[i].HolderType))

            if "SH" == arGdh[i].Market:
                shgdh = arGdh[i].SecuAccount
            if "SZ" == arGdh[i].Market:
                szgdh = arGdh[i].SecuAccount
    return szgdh, shgdh


def querydrwt_example(khh):
    """ 查询当日委托 """
    print ("=========== querydrwt_example ============")
    size, arDrwt = SECITPDK.QueryOrders(khh, 0, SORT_TYPE_AES, 0, 0, "", "", 0)
    if size < 0:
        retcode, retmsg = SECITPDK.GetLastError()
        cont.error("Query drwt failed, {}: {}".format(retcode, retmsg))
    else:
        print ("Query drwt success, result num:", size)
        for i in range(size):
            if i > 2:
                break
            print ("AccountId: {}, OrderId: {}, CXOrderId:{}, SBWTH:{}, KFSBDBH: {}, Market:{}, StockCode: {}, StockType:{}, EntrustType:{}, OrderPrice: {}, OrderQty: {}, MatchPrice: {}, MatchQty: {}, SecuAccount:{}, OrderStatus:{}, ResultInfo: {}, BrowIndex:{}".format(arDrwt[i].AccountId, arDrwt[i].OrderId, arDrwt[i].CXOrderId, arDrwt[i].SBWTH, arDrwt[i].KFSBDBH, arDrwt[i].Market, arDrwt[i].StockCode, arDrwt[i].StockType, arDrwt[i].EntrustType, arDrwt[i].OrderPrice, arDrwt[i].OrderQty, arDrwt[i].MatchPrice, arDrwt[i].MatchQty, arDrwt[i].SecuAccount, arDrwt[i].OrderStatus, arDrwt[i].ResultInfo, arDrwt[i].BrowIndex))


def querysscj_example(khh):
    """ 查询实时成交 """
    print ("=========== querysscj_example ============")
    size, arSSCJ = SECITPDK.QueryMatchs(khh, 0, SORT_TYPE_AES, 0, 0, "", "", 0)
    if size < 0:
        retcode, retmsg = SECITPDK.GetLastError()
        cont.error("Query sscj failed, {}: {}".format(retcode, retmsg))
    else:
        print ("Query sscj success, result num:", size)
        for i in range(size):
            if i > 2:
                break
            print ("AccountId: {}, Market:{}, SecuAccount:{}, EntrustType:{}, OrderId: {}, KFSBDBH: {}, MatchSerialNo: {}, StockCode: {}, MatchTime: {}, MatchPrice: {}, MatchQty: {}, ClearBalance: {}, BrowIndex:{}".format(arSSCJ[i].AccountId, arSSCJ[i].Market, arSSCJ[i].SecuAccount, arSSCJ[i].EntrustType, arSSCJ[i].OrderId, arSSCJ[i].KFSBDBH, arSSCJ[i].MatchSerialNo, arSSCJ[i].StockCode, arSSCJ[i].MatchTime, arSSCJ[i].MatchPrice, arSSCJ[i].MatchQty, arSSCJ[i].ClearBalance, arSSCJ[i].BrowIndex))


def queryzqgl(khh):
    """ 查询持仓 """
    #print ("=========== queryzqgl_example ============")
    size, arZQGL = SECITPDK.QueryPositions(khh, SORT_TYPE_AES, 0, 0, "", "", "", 1)
    if size < 0:
        retcode, retmsg = SECITPDK.GetLastError()
        cont.error("查询持仓失败, {}: {}".format(retcode, retmsg))
    else:
        print ("Query zqgl success, result num:", size)
        for i in range(size):
            if i > 2:
                break
            print ("AccountId: {}, SecuAccount: {}, StockCode: {}, CurrentQty: {}, MarketValue: {}, CostPrice: {}, DiluteCostPrice: {}, KeepCostPrice: {}, QtyAvl: {}".format(arZQGL[i].AccountId, arZQGL[i].SecuAccount, arZQGL[i].StockCode, arZQGL[i].CurrentQty, arZQGL[i].MarketValue, arZQGL[i].CostPrice, arZQGL[i].DiluteCostPrice, arZQGL[i].KeepCostPrice, arZQGL[i].QtyAvl))

def QueryFreezeDetails_example(khh):
    """ 查询资金冻结 """
    print ("=========== QueryFreezeDetails_example ============")
    size, arZJLS = SECITPDK.QueryFreezeDetails(khh, khh, 0, 0, 0)
    if size < 0:
        retcode, retmsg = SECITPDK.GetLastError()
        cont.error("Query ZJLS failed, {}: {}".format(retcode, retmsg))
    else:
        print ("Query ZJLS success, result num:", size)
        for i in range(size):
            if i > 2:
                break
            print ("AccountId: {}, FundAccount: {}, SerialNo: {}, MoneyType: {}, OccurDate: {}, FrozenType: {}, FrozenBalance: {}, ApplyTime: {}, Summary: {}".format(arZJLS[i].AccountId, arZJLS[i].FundAccount, arZJLS[i].SerialNo, arZJLS[i].MoneyType, arZJLS[i].OccurDate, arZJLS[i].FrozenType, arZJLS[i].FrozenBalance, arZJLS[i].ApplyTime, arZJLS[i].Summary))

def FundTransDetail_example(khh,pwd):
    """ 查询柜台资金流水 """
    print ("=========== FundTransDetail_example ============")
    size, arGTZJLS = SECITPDK.FundTransDetail(khh, pwd, "", 0, "")
    if size < 0:
        retcode, retmsg = SECITPDK.GetLastError()
        cont.error("Query FundTransDetail failed, {}: {}".format(retcode, retmsg))
    else:
        print ("Query FundTransDetail success, result num:", size)
        for i in range(size):
            if i > 2:
                break
            print ("AccountId: {}, FundAccount: {}, MoneyType: {}, ApplyTime: {}, FrozenBalance:{}, Summary: {}, BrowIndex: {}".format(arGTZJLS[i].AccountId, arGTZJLS[i].FundAccount, arGTZJLS[i].MoneyType, arGTZJLS[i].ApplyTime, arGTZJLS[i].FrozenBalance, arGTZJLS[i].Summary, arGTZJLS[i].BrowIndex))

def QueryZJHBCL_example(khh,pwd):
    """ 查询资金划拨策略 """
    print ("=========== QueryZJHBCL_example ============")
    size, arZJHBCL = SECITPDK.QueryZJHBCL(khh, pwd, khh)
    if size < 0:
        retcode, retmsg = SECITPDK.GetLastError()
        cont.error("Query QueryZJHBCL failed, {}: {}".format(retcode, retmsg))
    else:
        print ("Query QueryZJHBCL success, result num:", size)
        for i in range(size):
            if i > 2:
                break
            print ("AccountId: {}, FundAccount: {}, MoneyType: {}, NodeId: {}, Market:{}, Rate: {}".format(arZJHBCL[i].AccountId, arZJHBCL[i].FundAccount, arZJHBCL[i].MoneyType, arZJHBCL[i].NodeId, arZJHBCL[i].Market, arZJHBCL[i].Rate))

def QueryJDJZJHBMX_example(khh,pwd):
    """ 查询节点间资金划拨明细 """
    print ("=========== QueryJDJZJHBMX_example ============")
    size, arJDJZJHBMX = SECITPDK.QueryJDJZJHBMX(khh, pwd, "", 0, "")
    if size < 0:
        retcode, retmsg = SECITPDK.GetLastError()
        cont.error("Query QueryJDJZJHBMX failed, {}: {}".format(retcode, retmsg))
    else:
        print ("Query QueryJDJZJHBMX success, result num:", size)
        for i in range(size):
            if i > 2:
                break
            print ("AccountId: {}, FundAccount: {}, MoneyType: {}, NodeId1: {}, NodeId2:{}, TradeDate:{}, TransDate:{}, Summary: {}, ApplyTime:{}, OccurAmt:{}, BrowIndex: {}".format(arJDJZJHBMX[i].AccountId, arJDJZJHBMX[i].FundAccount, arJDJZJHBMX[i].MoneyType, arJDJZJHBMX[i].NodeId1, arJDJZJHBMX[i].NodeId2, arJDJZJHBMX[i].TradeDate, arJDJZJHBMX[i].TransDate, arJDJZJHBMX[i].ApplyTime, arJDJZJHBMX[i].OccurAmt, arJDJZJHBMX[i].Summary, arJDJZJHBMX[i].BrowIndex))

def querypsqy_example(khh):
    """ 查询配售权益 """
    print ("=========== querypsqy_example ============")
    size, arPSQY = SECITPDK.QueryPSQY(khh, "", "")
    if size < 0:
        retcode, retmsg = SECITPDK.GetLastError()
        cont.error("Query psqy failed, {}: {}".format(retcode, retmsg))
    else:
        print ("Query psqy success, result num:", size)
        for i in range(size):
            if i > 2:
                break
            print ("AccountId: {}, SecuAccount: {}, Market: {}, BallotQty: {}, StartQty: {}".format(arPSQY[i].AccountId, arPSQY[i].SecuAccount, arPSQY[i].Market, arPSQY[i].BallotQty, arPSQY[i].StartQty))

def queryZQDM_example(khh):
    """ 查询证券代码 """
    print ("=========== queryZQDM_example ============")
    size, arZQDM = SECITPDK.QueryZQDMInfo(khh, 0, "", "SZ", "", "", 0)
    if size < 0:
        retcode, retmsg = SECITPDK.GetLastError()
        cont.error("Query ZQDM failed, {}: {}".format(retcode, retmsg))
    else:
        print ("Query ZQDM success, result num:", size)
        for i in range(size):
            if i > 2:
                break
            print ("StockCode: {}, StockName: {}, StockType:{}, Market: {}, MaxTradeAmt:{}, MinTradeAmt:{}".format(arZQDM[i].StockCode, arZQDM[i].StockName, arZQDM[i].StockType, arZQDM[i].Market, arZQDM[i].MaxTradeAmt, arZQDM[i].MinTradeAmt))

def queryTPXX_example(khh):
    """ 查询网络投票信息 """
    print ("=========== queryTPXX_example ============")
    size, arTPXX = SECITPDK.QueryTPXX(khh, 0, 0, "", "")
    if size < 0:
        retcode, retmsg = SECITPDK.GetLastError()
        cont.error("Query TPXX failed, {}: {}".format(retcode, retmsg))
    else:
        print ("Query TPXX success, result num:", size)
        for i in range(size):
            if i > 2:
                break
            print ("Market: {}, StockCode: {}, StockType: {}, CompanyCode: {}, MeetingCode:{}, MeetingName: {}, MotionCode:{}, MotionName:{}, MotionType:{}, BrowIndex: {}".format(arTPXX[i].Market, arTPXX[i].StockCode, arTPXX[i].StockType, arTPXX[i].CompanyCode, arTPXX[i].MeetingCode, arTPXX[i].MeetingName, arTPXX[i].MotionCode, arTPXX[i].MotionName, arTPXX[i].MotionType, arTPXX[i].BrowIndex))

def queryetfmx_example(khh):
    """ 查询ETTF成分股信息 """
    print ("=========== queryetfmx_example ============")
    size, arEtfmx = SECITPDK.QueryETFShare(khh, "510050")
    if size < 0:
        retcode, retmsg = SECITPDK.GetLastError()
        cont.error("Query etfmx failed, {}: {}".format(retcode, retmsg))
    else:
        print ("Query etfmx success, result num:", size)
        for i in range(size):
            if i > 2:
                break
            print ("FundCode: {}, Market: {}, StockCode: {}, ComponentQty: {}, PremiumRatio: {}, CashSubstitute: {}, SubstituteFlag: {}".format(arEtfmx[i].FundCode, arEtfmx[i].Market, arEtfmx[i].StockCode, arEtfmx[i].ComponentQty, arEtfmx[i].PremiumRatio, arEtfmx[i].CashSubstitute, arEtfmx[i].SubstituteFlag))


def orderenstrust_example(khh, szgdh, shgdh):
    """ 普通买卖 """
    print ("=========== orderenstrust_example ============")
    kmsl = SECITPDK.TradableQty(khh, "SZ", "000001", JYLB_BUY, 10.1, DDLX_XJWT, szgdh)
    if kmsl >= 0:
        print ("Tradeable qty =", kmsl)
    else:
        retcode, retmsg = SECITPDK.GetLastError()
        cont.error("TradableQty failed, {}: {}".format(retcode, retmsg))

    wth = SECITPDK.OrderEntrust(khh, "SZ", "000001", JYLB_BUY, 2000, 10.1, DDLX_XJWT, szgdh)
    if wth > 0:
        print ("Order orderenstrust success, wth=", wth)
        # 撤单
        cdwth = SECITPDK.OrderWithdraw(khh, "SZ", wth)
        if cdwth > 0:
            print ("Order withdraw success, wth=", wth)
        else:
            retcode, retmsg = SECITPDK.GetLastError()
            cont.error("Order withdraw failed, {}: {}".format(retcode, retmsg))
        # 撤单（异步）
        # ret = SECITPDK.OrderWithdraw_ASync(khh, "SZ", wth)
        # if ret > 0:
        #     print "Order withdraw async success, kfsbdbh=", ret
        # else:
        #     retcode, retmsg = SECITPDK.GetLastError()
        #     cont.error("Order withdraw async failed, {}: {}".format(retcode, retmsg))
    else:
        retcode, retmsg = SECITPDK.GetLastError()
        cont.error("Order failed, {}: {}".format(retcode, retmsg))


def orderenstrust_async_example(khh, szgdh, shgdh):
    """ 异步普通买卖 """
    print ("=========== orderenstrust_async_example ============")
    kfsbdbh = SECITPDK.OrderEntrust_ASync(khh, "SZ", "000001", JYLB_BUY, 100, 10.1, DDLX_XJWT, szgdh)
    if kfsbdbh > 0:
        print ("Order orderenstrust async success, kfsbdbh=", kfsbdbh)
        # 根据开发商本地编号撤单
        cdwth = SECITPDK.OrderWithdrawByKFSBDBH(khh, "SZ", kfsbdbh)
        if cdwth > 0:
            print ("Order withdraw by kfsbdbh success, wth=", cdwth)
        else:
            retcode, retmsg = SECITPDK.GetLastError()
            cont.error("Order withdraw by kfsbdbh failed, {}: {}".format(retcode, retmsg))
        # 根据开发商本地编号撤单（异步）
        # ret = SECITPDK.OrderWithdrawByKFSBDBH_ASync(khh, "SZ", kfsbdbh)
        # if ret > 0:
        #     print "Order withdraw by kfsbdbh async success, kfsbdbh=", ret
        # else:
        #     retcode, retmsg = SECITPDK.GetLastError()
        #     cont.error("Order withdraw by kfsbdbh async failed, {}: {}".format(retcode, retmsg))
    else:
        retcode, retmsg = SECITPDK.GetLastError()
        cont.error("Order failed, {}: {}".format(retcode, retmsg))


def batchorderenstrust_example(khh, szgdh, shgdh):
    """ 普通买卖批量委托 """
    print ("=========== batchorderenstrust_example ============")
    batchorder = (ITPDK_BatchOrder*3)()
    for i in range(len(batchorder)):
        batchorder[i].Market = "SZ"
        if 0 == i:
            batchorder[i].StockCode = "000001"
        else:
            batchorder[i].StockCode = "000002"
        batchorder[i].EntrustType = JYLB_BUY
        batchorder[i].OrderPrice = 10.1
        batchorder[i].OrderQty = 100
        batchorder[i].OrderType = DDLX_XJWT
        batchorder[i].SecuAccount = szgdh

    ret = SECITPDK.BatchOrderEntrust(khh, batchorder, 1)
    if ret >= 0:
        print ("Batch order success {}/{}".format(len(batchorder)-ret, len(batchorder)))
        for i in range(len(batchorder)):
            print ("No.{} -- WTH: {}, RETMESSAGE: {}".format(i+1, batchorder[i].OrderId, batchorder[i].RetMessage))
    else:
        retcode, retmsg = SECITPDK.GetLastError()
        cont.error("Batch order failed, {}: {}".format(retcode, retmsg))


def batchorderenstrust_async_example(khh, szgdh, shgdh):
    """ 异步普通买卖批量委托 """
    print ("=========== batchorderenstrust_async_example ============")
    batchorder = (ITPDK_BatchOrder*3)()
    for i in range(len(batchorder)):
        batchorder[i].Market = "SZ"
        batchorder[i].StockCode = "000001"
        batchorder[i].EntrustType = JYLB_BUY
        batchorder[i].OrderPrice = 10.1
        batchorder[i].OrderQty = 100
        batchorder[i].OrderType = DDLX_XJWT
        batchorder[i].SecuAccount = szgdh

    ret = SECITPDK.BatchOrderEntrust_ASync(khh, batchorder, 1)
    if ret >= 0:
        print ("Batch order async success")
    else:
        retcode, retmsg = SECITPDK.GetLastError()
        cont.error("Batch order async failed, {}: {}".format(retcode, retmsg))


def etfenstrust_example(khh, szgdh, shgdh):
    """ ETF申赎 """
    print ("=========== etfenstrust_example ============")
    kmsl = SECITPDK.ETFTradableQty(khh, "SZ", "159001", JYLB_ETFSG, szgdh)
    if kmsl >= 0:
        print ("ETFTradeable qty =", kmsl)
    else:
        retcode, retmsg = SECITPDK.GetLastError()
        cont.error("ETFTradableQty failed, {}: {}".format(retcode, retmsg))

    wth = SECITPDK.ETFEntrust(khh, "SZ", "159001", JYLB_ETFSG, 100, szgdh)
    if wth > 0:
        print ("Order etfenstrust success, wth=", wth)
        # 撤单
        cdwth = SECITPDK.ETFWithdraw(khh, "SZ", wth)
        if cdwth > 0:
            print ("Order withdraw success, wth=", wth)
        else:
            retcode, retmsg = SECITPDK.GetLastError()
            cont.error("Order withdraw failed, {}: {}".format(retcode, retmsg))
    else:
        retcode, retmsg = SECITPDK.GetLastError()
        cont.error("Order failed, {}: {}".format(retcode, retmsg))


def etfenstrust_async_example(khh, szgdh, shgdh):
    """ ETF申赎 """
    print ("=========== etfenstrust_async_example ============")
    kfsbdbh = SECITPDK.ETFEntrust_ASync(khh, "SZ", "159001", JYLB_ETFSG, 100, szgdh)
    if kfsbdbh > 0:
        print ("Order etfenstrust success, kfsbdbh=", kfsbdbh)
    else:
        retcode, retmsg = SECITPDK.GetLastError()
        cont.error("Order failed, {}: {}".format(retcode, retmsg))


def lofenstrust_example(khh, szgdh, shgdh):
    """ LOF基金 """
    print ("=========== lofenstrust_example ============")
    kmsl = SECITPDK.LOFTradableQty(khh, "SH", "501093", JYLB_JJRG, 10.1, DDLX_XJWT, shgdh)
    if kmsl >= 0:
        print ("LOFTradeable qty =", kmsl)
    else:
        retcode, retmsg = SECITPDK.GetLastError()
        cont.error("LOFTradableQty failed, {}: {}".format(retcode, retmsg))

    wth = SECITPDK.LOFEntrust(khh, "SH", "501093", JYLB_JJRG, 10000, 10.1, DDLX_XJWT, shgdh)
    if wth > 0:
        print ("Order lofenstrust success, wth=", wth)
        # # 撤单
        # cdwth = SECITPDK.ETFWithdraw(khh, "SZ", wth)
        # if cdwth > 0:
        #     print "Order withdraw success, wth=", wth
        # else:
        #     retcode, retmsg = SECITPDK.GetLastError()
        #     cont.error("Order withdraw failed, {}: {}".format(retcode, retmsg))
    else:
        retcode, retmsg = SECITPDK.GetLastError()
        cont.error("Order failed, {}: {}".format(retcode, retmsg))

def shjjtenstrust_example(khh, szgdh, shgdh):
    """ 上海基金通 """
    wth = SECITPDK.SHJJT(khh, "SH", "519001", JYLB_JJSG, 100, 10.1, DDLX_XJWT, 0, "", shgdh)
    if wth > 0:
        print ("Order shjjtenstrust success, wth=", wth)
        # # 撤单
        # cdwth = SECITPDK.ETFWithdraw(khh, "SZ", wth)
        # if cdwth > 0:
        #     print "Order withdraw success, wth=", wth
        # else:
        #     retcode, retmsg = SECITPDK.GetLastError()
        #     cont.error("Order withdraw failed, {}: {}".format(retcode, retmsg))
    else:
        retcode, retmsg = SECITPDK.GetLastError()
        cont.error("Order failed, {}: {}".format(retcode, retmsg))





def runHtsRealTraderProcess(user, account, classname, strats,traderqueue,user_setting):
    '''
    运行实盘交易进程
    :param user: 用户名
    :param account: 账户名
    :param classname: 现货/期权/两融
    :param strats: 策略实例
    :param traderqueue: 交易队列
    :param user_setting: 用户设置
        investorid:投资者代码
        password:密码
        brokerid:经纪商代码key
    :return:
    '''
    global curHtsTrader

    # python .\secdemo.py -key A5_RS_115 -khh 221188943404 -pwd 000783
    # python .\secdemo.py -key A5_RS_93 -khh 221188943403 -pwd 123123
    # python .\secdemo.py -key A5_RS_156 -khh 000001 -pwd 000783
    # python .\secdemo.py -key A5_RS_199 -khh 200900138524 -pwd 000783
    '''
    parser = argparse.ArgumentParser()
    parser.add_argument("-key", "--key",     help="login key.",  type=str, default="")
    parser.add_argument("-khh", "--khh",     help="khh.",  type=str, default="")
    parser.add_argument("-pwd", "--pwd",       help="pwd.",  type=str, default="")
    parser.add_argument("-wtfs", "--wtfs",     help="wtfs.",  type=str, default="32")
    args = parser.parse_args()
    '''

    try:
        key = user_setting['brokerid']
        khh = user_setting['investorid']
        pwd = user_setting['password']
        wtfs = "32"
    except Exception as e:
        print (e)
        return
    curdir = os.path.dirname(os.path.abspath(__file__))
    SECITPDK.SetLogPath("")
    print('profile dir',curdir)
    SECITPDK.SetProfilePath(curdir+'/secitpdk/')

    # init
    SECITPDK.Init()
    SECITPDK.SetWriteLog(True)
    SECITPDK.SetFixWriteLog(False)

    curHtsTrader = HtsTrader(user,account,classname,strats,traderqueue,khh)
    
    ## Set callback functions
    SECITPDK.SetFuncCallback(curHtsTrader.on_func_call_back)
    SECITPDK.SetMsgCallback(curHtsTrader.on_msg_call_back)
    SECITPDK.SetConnEventCallback(curHtsTrader.on_connevent_call_back)
    
    
    ## Login to server
    SECITPDK.SetWTFS(wtfs)
    SECITPDK.SetNode("pysecitpdk")
    token = SECITPDK.TradeLogin(key, khh, pwd)
    if token > 0:
        print ("Login success, token:", token)
        curHtsTrader.TraderProcess()
    else:
        retcode, retmsg = SECITPDK.GetLastError()
        cont.error("Login error, {}: {}".format(retcode, retmsg))

def test_trader(khh):
    # query
    # queryfundinfo_example(khh)
    # QueryFreezeDetails_example(khh)
    # queryZQDM_example(khh)
    queryTPXX_example(khh)
    # FundTransDetail_example(khh)
    # QueryZJHBCL_example(khh)
    # QueryJDJZJHBMX_example(khh)
    szgdh, shgdh = queryaccinfo_example(khh)
    # querynodeinfo_example(key,khh)
    # querydrwt_example(khh)
    # querysscj_example(khh)
    # queryzqgl_example(khh)
    # querypsqy_example(khh)
    # queryetfmx_example(khh)

    # trade
    orderenstrust_example(khh, szgdh, shgdh)
    # orderenstrust_async_example(khh, szgdh, shgdh)
    # batchorderenstrust_example(khh, szgdh, shgdh)
    # batchorderenstrust_async_example(khh, szgdh, shgdh)
    # etfenstrust_example(khh, szgdh, shgdh)
    # etfenstrust_async_example(khh, szgdh, shgdh)
    # lofenstrust_example(khh, szgdh, shgdh)
    # shjjtenstrust_example(khh, szgdh, shgdh)

    # wth = []
    # for i in range(2):
    #     ret = SECITPDK.OrderEntrust(khh, "SH", "600000", JYLB_BUY, 101300, 16, DDLX_XJWT, shgdh)
    #     if ret < 0:
    #         retcode, retmsg = SECITPDK.GetLastError()
    #         cont.error("Order by kfsbdbh async failed, {}: {}".format(retcode, retmsg))
    #     else:
    #         cont.ok("Order success, wth={}".format(ret))
    #         wth.append(ret)

    # time.sleep(20)
    # for i in wth:
    #     ret = SECITPDK.OrderWithdraw(khh, " ", i)
    #     if ret < 0:
    #         retcode, retmsg = SECITPDK.GetLastError()
    #         cont.error("Order withdraw by kfsbdbh async failed, {}: {}".format(retcode, retmsg))
    #     else:
    #         cont.ok("Order withdraw success, wth={}".format(ret))
    #         wth.append(ret)


        

    # for j in range(200):
    #     for i in range(5000):
    #         ret = SECITPDK.OrderEntrust_ASync(khh, "SZ", "000001", JYLB_BUY, 200, 14, DDLX_XJWT, szgdh)
    #         if ret < 0:
    #             retcode, retmsg = SECITPDK.GetLastError()
    #             cont.error("Order withdraw by kfsbdbh async failed, {}: {}".format(retcode, retmsg))
    #     time.sleep(1)

    # for i in range(100000):
    #     size, arSSCJ = SECITPDK.QueryMatchs(khh, 0, SORT_TYPE_AES, 0, 0, "", "", 0)
    #     if size < 0:
    #         retcode, retmsg = SECITPDK.GetLastError()
    #         cont.error("Query sscj failed, {}: {}".format(retcode, retmsg))

    # for i in range(100000):
    #     size, arDrwt = SECITPDK.QueryOrders(khh, 0, SORT_TYPE_DESC, 0, 0, "", "", 0)
    #     if size < 0:
    #         retcode, retmsg = SECITPDK.GetLastError()
    #         cont.error("Query drwt failed, {}: {}".format(retcode, retmsg))


    # for i in range(20):
    #     wth = SECITPDK.ETFEntrust(khh, "SH", "511651", JYLB_ETFSG, 1000000, shgdh)
    #     if wth > 0:
    #         print "Order etfenstrust success, wth=", wth
    #         # 撤单
    #         # cdwth = SECITPDK.ETFWithdraw(khh, "SZ", wth)
    #         # if cdwth > 0:
    #         #     print "Order withdraw success, wth=", wth
    #         # else:
    #         #     retcode, retmsg = SECITPDK.GetLastError()
    #         #     cont.error("Order withdraw failed, {}: {}".format(retcode, retmsg))
    #     else:
    #         retcode, retmsg = SECITPDK.GetLastError()
    #         cont.error("Order failed, {}: {}".format(retcode, retmsg))


    # test
    # for i in range(1):
    #     for j in range(300):
    #         # orderenstrust_async_example()
    #         ret = SECITPDK.OrderEntrust(khh, "SH", "600000", JYLB_BUY, 200, 10.1, DDLX_XJWT, shgdh)
    #         if ret < 0:
    #             retcode, retmsg = SECITPDK.GetLastError()
    #             cont.error("Order withdraw by kfsbdbh async failed, {}: {}".format(retcode, retmsg))

    #     time.sleep(1)
    # for i in range(100):
    #     size, arDrwt = SECITPDK.QueryOrders(khh, 0, SORT_TYPE_DESC, 0, 0, "", "", 0)
    #     if size < 0:
    #         retcode, retmsg = SECITPDK.GetLastError()
    #         cont.error("Query drwt failed, {}: {}".format(retcode, retmsg))
    #     else:
    #         for i in range(size):
    #             if i > 2:
    #                 break
    #             print "AccountId: {}, OrderId: {}, KFSBDBH: {}, StockCode: {}, OrderPrice: {}, OrderQty: {}, MatchPrice: {}, MatchQty: {}, ResultInfo: {}".format(arDrwt[i].AccountId, arDrwt[i].OrderId, arDrwt[i].KFSBDBH, arDrwt[i].StockCode, arDrwt[i].OrderPrice, arDrwt[i].OrderQty, arDrwt[i].MatchPrice, arDrwt[i].MatchQty, arDrwt[i].ResultInfo)
    # for i in range(100):
    #     size, arSSCJ = SECITPDK.QueryMatchs(khh, 0, SORT_TYPE_AES, 0, 0, "SZ", "000001", 0)
    #     if size < 0:
    #         retcode, retmsg = SECITPDK.GetLastError()
    #         cont.error("Query drwt failed, {}: {}".format(retcode, retmsg))
    #     else:
    #         for i in range(size):
    #             if i > 2:
    #                 break
    #             print "AccountId: {}, OrderId: {}, KFSBDBH: {}, StockCode: {}, MatchPrice: {}, MatchQty: {}, BrowIndex: {}".format(arSSCJ[i].AccountId, arSSCJ[i].OrderId, arSSCJ[i].KFSBDBH, arSSCJ[i].StockCode, arSSCJ[i].MatchPrice, arSSCJ[i].MatchQty, arSSCJ[i].BrowIndex)
    
    # for i in range(31):
    #     size, arZQGL = SECITPDK.QueryPositions(khh, SORT_TYPE_AES, 0, 0, "", "", "", 0)
    #     if size < 0:
    #         retcode, retmsg = SECITPDK.GetLastError()
    #         cont.error("Query zqgl failed, {}: {}".format(retcode, retmsg))


    # wth = SECITPDK.LOFEntrust(khh, "SZ", "160224", JYLB_JJFC, 100, 10.1, DDLX_XJWT, szgdh)
    # if wth > 0:
    #     print "Order lofenstrust success, wth=", wth
    # else:
    #     retcode, retmsg = SECITPDK.GetLastError()
    #     cont.error("Order failed, {}: {}".format(retcode, retmsg))

    # exit
    time.sleep(30)
    SECITPDK.Exit()
    cont.ok("Test complete!")

def testHTSTrader():
    from multiprocessing import Queue
    user = 'test'
    from .qeaccount import realAccountInfo

    account = realAccountInfo()
    classname = 'stock'
    strats = ['test']
    traderqueue = Queue()
    user_setting = {'investorid':"2005255",'password':"202123", 'brokerid':"A5_RS_156_1"}
    runHtsRealTraderProcess(user, account, classname, strats, traderqueue, user_setting)